CATCosinor <-
function (TimeCol=1,Y=2, Components=1, window="noTaper", RefDateTime=NA,  timeFormat="%Y%m%d%H%M", RangeDateTime=list(Start=NA, End=NA), Units="hour", dt=0, Progressive=list(Interval=144, Increment=8), Period=list(Set=0,Start=0,Increment=1,End=0),header=F, Skip=0, Colors="BW",Graphics="pdf",Output=list(Txt=F,Dat=T, Doc=T,Graphs=F),Console=F,Debug=FALSE,fileName=fileName,functionName="") {    #GraphSet is another parameter used to override proper graphics
  # NAs are interpolated;  0s are modified to a very close to 0 value as some functions will not accept 0 data values.
  # Header:  T/F to indicate if the file has a header  (column titles for data)
  # Skip:  How many lines to skip at the top of the file (before the header)
  # Console:  If T, all data is sent to R console.  Otherwise, sent to file type indicated in Graphics parameter
  #  Notes on coding for time:
  #         %H represents 0-23 hrs, so have to have numeric time in Excel sheet, and numeric time rep option
  #         Dates are represented as the number of days since 1970-01-01
  #         strptime reads in characters and produces POSIX formatted date/time;  strptime accepts the format of date being read -- output is always POSIX
  #         use format wrapped around POSIX to output a date in a different format than POSIX
  #         POSIXct: This class stores a date in a vector, as the number of seconds passed since January, 1st, 1970 at 1AM
  #         POSIXlt: This class stores a date in a list: (day of the month, day of the week, day of the year, month, year, time zone, hour, minute, second).
  # TimeCol:    Column(s) where time is found.  c(1,2) can be used when date is in column 1 and time is in column 2.  Date and time can be in any two columns, i.e., c(6,3). 
  #             If time is in two columns, the formats should be %d/%m/%y in the first of the two columns, and %H:%M:%S in the second of the two
  #             (Any time format in timeFormat paramater is ignored in this case.)
  # Y:          Column holding data to be analyzed
  # Components:  Default=1.  Indicates if this is a single or multiple component cosinor analysis, where the number of components is specified (>0).  
  # window:      Data will be convolved with a window:  Hanning, Hamming, Bartlett, Blackman.  Default="noTaper"
  # RefDateTime:  Date used as reference, and subtracted from all data dates, to make the number smaller.  Default=NA
  #             not implemented---if RefDateTime = NA, use the 1st date of the data as the RefDateTime 
  #             if RefDateTime = 0, use midnight of the same day as the data starts 
  #             when using years with leading 0s  (0500 AD) be sure to include the leading 0, and put quotes around the RefDateTime
  # timeFormat:   RefDateTime and the time in the data file should both use the same date/time format.   Can be "numeric" or "%Y%m%d%H%M"
  #             i.e., "%Y%m%d%H%M", or if = "numeric", time column in data file can be simple numbers (0 - 99999...)
  #               Time can also be in two columns (indicate in TimeCol);  timeFormat is ignored when time is in two columns -- the format use is %d/%m/%y in the first of the two columns, and %H:%M:%S or %H:%M in the second of the two
  # RangeDateTime
  #      $Start:  Analysis is started at this date/time.  May be before 1st data date.  Default = NA
  #             if Start = NA, use the 1st time in the data set
  #             if Start = 0 use Midnight at the start of the 1st date of the data as the StartDate 
  #        $End:    Analysis ends at this date/time.  May be after last data date.
  #             if End = NA, use the last time in the data set
  #             if End = 0 use the midnight at the end of the last date of the data as the EndDate   
  # Units:        Units (hour, year, week or day) for Interval and Increment arguments, as well as Period arguments, and any time variable
  #               ToDo:  modify to use units other than hours -- currently only works for hours
  # Progressive
  #   $Interval : length of the time span being analyzed (in Units)  -- multiple spans calculated
  #               If 0, assumes no progession, Interval is set to the full dataset length, and Increment = full data set  
  #  $Increment: number of Days, Wks or Yrs  (uses same unit as set for Interval) to shift forward for each successive Interval analyses
  #               If 0, assumes no progession, Interval is set to the full dataset length, and Increment = full data set 
  # Period
  #       $End :  [only used if $Set=0] Last (and smallest) period to calculate, in units of Days, Wks or Yrs (as set by Units)  EXCLUSIVE
  #               Defaults to 2*dt or 4;  (1 is too small) 0 is invalid  -- default will be used
  #  $Increment : [only used if $Set=0] Increment to starting period, in units of Days, Wks or Yrs (as set by Units)
  #               Defaults to 1;   0 is invalid  -- default will be used
  #     $Start :  [only used if $Set=0] First (and largest) period to calculate, in units of Days, Wks or Yrs (as set by Units);  (Interval/1)
  #               0 is Default: the full time range of the data file is analyzed [in hours?] (RangeDateTime$End-RangeDateTime$Start)= MyData_length; or if progressive, Interval/1;     
  #               Important:  It is normally best if the user sets $Start to a multiple of the largest period of interest  -- Fourier frequencies calculated will be closest to period of interest this way.
  #                           So if one is interested in a weekly period, use a multiple of 1 week as $Start
  #      $Set :  If Set=0, a series of periods are analyzed (spectrum) according to Period$Start, $Increment, $End (or their default value, if not specified)
  #              If not equal to 0, Overrides Period$Start and $Increment, to completely specify an exact set of periods to analyze (as set by Units).  
  #              Can be in the format of a single number, or an array:  c(1,2,3) or c(8,12:24)
  #              When Components=1, each period specified in the vector will be assessed by cosinor independently.
  #              When parameter Components is >1, Period$Set must have a correspondig number of components, which assessed together in a multiple component cosinor.
  #              When 0, only the maximum period, or the maximum period per Interval, from a spectrum is listed on p1 of the graphics file.
  #              Otherwise, all periods are displayed on the graphic 
  #  Colors       "Heat" renders the heat map in colors;  "BW" renders the heatmap in grayscale
  #  Graphics    Possible values:  jpg, pdf, tif, png
  #  Output      $Txt will capture the console output to a .txt file
  #              $Dat will generate a computer readable file of data in a not very human friendly form, but nice for subsequent processing (in a Mean Population Cosinor for example)
  #              $Doc will generate a nicely formatted Word doc
  #              $Graphs will enable a set of graphs plotting Data, Model, MESOR, Amplitude, Acrophase over time--too many when you do a progressive
  #  GraphSet:    This is an undocumented parameter.  Defaults are used normally, but can be overriden by this parameter
  #               This will set which line graphs are plotted:  Data, Model, MESOR, Amplitude, Acrophase over time            
  #               $Data plots the (filtered) data;  $Model plots the Cosinor model itself;  
  #               $HeatMap=will do a heatmap if enough Intervals and periods are specified for a progressive analysis,
  # dt           When equidistant data, dt indicates the sampling interval.  If dt =0, no periodogram is done.
  #                  Data is assumed to be equidistant when this is nonzero
  # Debug       Turn on when you want to see more diagnostic output for programming debug purposes
  # fileName    Name of the file containing input data
  # functionName:  Name given the run, to help distinguish it from other runs.  Name is printed in output files.
   
  #library(season)

  #####################################################################################################
  #                                                                                                   #
  #        Set up some initial parameter values, error messages, and key variables                    #
  #                                                                                                   #
  #####################################################################################################
  tz="GMT"
  opar = par()
    # read data;  if no data file name given, prompt the user for the filename
if (length(fileName)==0){
    fileName <- file.choose()
  }
# calculate greatest common divisor and least common multiple;  used later to calculate time in one degree
  if (Components>1){ 
  "%gcd%" <- function(u, v) {ifelse(u %% v != 0, v %gcd% (u%%v), v)}
  "%lcm%" <- function(u, v) { abs(u*v)/(u %gcd% v)}
  LCM<-1
    for (i in 1:Components){
      LCM<-Period$Set[i] %lcm% LCM
    }
  }
#  if not yet set, set defaults;  This should be added as an undocumented parameter to override which Graphs print by default
# if lss1 ..  else if non-progressive2 ... else if gliding spectrum progressive3 ... else if 1-comp progressive4 or multi-comp progressive5
#  raw data, model, MESOR, Amplitude, Phase, N (# pts), heatmap
if (!exists("GraphSet")){    
    if (Progressive$Interval==0 || Progressive$Increment==0){    #  default settings for non-progressives (2)
      if (Period$Set[1]==0){      # non-progressive lss (1)
      GraphSet<-list(Data=T,Model=F,MESOR=F,Amp=T,P=T,Phi=F,NPts=F, HeatMap=F)
      } else GraphSet<-list(Data=T,Model=T,MESOR=F,Amp=F,Phi=F,P=F,NPts=F, HeatMap=F)   #  (2) HeatMap will only be created in certain conditions determine later,
        #  progressives:
    } else {if (Period$Set[1]==0){     #  progressive lss  (3)
                 GraphSet<-list(Data=T,Model=F,MESOR=F,Amp=F,P=F,Phi=F,NPts=F, HeatMap=T)   #  raw data, MESOR, Amplitude, Phase, N (# pts)
                  } else GraphSet<-list(Data=T,Model=T,MESOR=T,Amp=T,P=T,Phi=T,NPts=T, HeatMap=F)    #  regular progressive (4)
            }  # end if period$set==0
  }

#   if (!exists("GraphSet")){    
#     if (Progressive$Interval==0 || Progressive$Increment==0){    #  default settings for non-progressives 
#       GraphSet<-list(Data=T,Model=T,MESOR=F,Amp=T,Phi=T,P=F,NPts=F, HeatMap=F)   #HeatMap will only be created in certain conditions determine later,
#     } else {if (Period$Set==0){
#                 GraphSet<-list(Data=T,Model=F,MESOR=F,Amp=F,P=F,Phi=F,NPts=F, HeatMap=T)   #  raw data, MESOR, Amplitude, Phase, N (# pts)
#               } else GraphSet<-list(Data=T,Model=T,MESOR=T,Amp=T,P=T,Phi=T,NPts=T, HeatMap=F) 
#           }  # end if period$set==0
#   }
  # rename parameters
  StartDate<-RangeDateTime$Start
  EndDate<-RangeDateTime$End
  Interval<-Progressive$Interval
  Increment<-Progressive$Increment
  FreqInc<-Period$Increment
  #Period$Start<-Period$Start
  #Period$End<-Period$End
  oneCycle<-Period$Set     # can be a vector of one or more
  Ys_end<-length(Y)
  paramMsg<-paste("\n  TimeCol=",TimeCol,",  Y=",Y, ",  header=",header,"\n --  Periods=",Period["Set"],", Units=",Units, ",  Interval=",format(Interval,nsmall=3), ",  Increment=",format(Increment,nsmall=3), "\nPeriod$Start=",format(Period$Start,nsmall=3), ",  FreqInc=",format(FreqInc,nsmall=3), ",  Period$End=",format(Period$End,nsmall=3), "\nRefDateTime=",RefDateTime, ", StartDate=",format(StartDate,nsmall=3),", EndDate=",format(EndDate,nsmall=3),"Percent of missing (blank) sample values: unknown","\n",functionName,"\n")

  CosMatrixDim<-Components*2+1         #  validate and error if too large -- should it be Components*2?  no Feb 26
  if (oneCycle[1]>0){
    if (Components>1 && length(oneCycle)!=Components){
      errMsg<-paste("Error:  When parameter Components is >1, Period$Set must have a corresponding number of components.  \nComponents=",Components,"; Period$Set=",Period$Set)
      closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
      stop(errMsg)
       # Period$Set will contain multiple possible components to check separately, or as a multi-component cosinor, depending on param Components
       # if Components=1, Period$Set can have any number of components.  If Components>1, length(oneCyce) must equal Components
    }
  }
  errMsg<-""            #   this error message gets printed at the beginning of the rtf output file
  #  symbols currently in use for Err values, printed in the matrix
  errKey<-c("*","**","+","++","@","*@","@@@","@@")   
# messages that correspond to the errKey symbols
  keyMsgs<- vector(mode="character",length = 10)
  keyMsgs[1]<-paste("*  : Error1:  The matrix cannot be inverted for these points and period. This may happen if there are not enough data points for the period selected. Check sampling interval: there should be at least 2p+1 data points per period.\n")
  keyMsgs[2]<-paste("** : Error2:  The matrix cannot be inverted for these points and period. This may happen if there are not enough data points for the period selected. Check sampling interval: there should be at least 2p+1 data points per period.\n")
  keyMsgs[3]<-"+  : Error:  This model does not allow calculation of PR for the individual components. PR is negative, which is invalid.\n"
  keyMsgs[4]<-"++ : Error:  Coefs cannot be calculated\n"        #  this one is likely not needed
  keyMsgs[5]<-"@  : Error:  Requested analysis violates Nyquist.  Time span should be > 2C, where C=#cycles.  Results may be invalid.\n"
  keyMsgs[6]<-"*@ : Warning:  RSS=0.  Not enough data to calculate S.E.  \n"
  #  if there is a period of 8 hours being assessed, there must be 4 data points in 8 hours.
  keyMsgs[7]<-"@@@: Error:  This interval must be at least as long as the trial period (Period$Set).  No analysis performed when Interval<90% of target period.\n"
  keyMsgs[8]<-"@@: Warning:  The interval must be at least as long as the trial period (Period$Set).  Results may be unreliable.\n"


  # extract filename from path for display on output 
  if (.Platform$file.sep=="/"){
    m=regexec("[^/]*$",fileName)            #  ./(.$)    pc=[^\\]*$   mac=[^/]*$
  } else {
    m=regexec("[^\\]*$",fileName)
  }
  fileName1<-regmatches(fileName,m) 
  fileLen<-nchar(fileName1)
  fileName6<-substring(fileName1,1,fileLen-4)
  # send output to file  -- build name of file with path to data f
  BaseTime<-Sys.time()
  thisTime <- format(BaseTime, "--%d%b%y--%H-%M-%OS")
  fileName2<-paste(fileName,window,thisTime,functionName,"Cos.txt",sep="")
  fileName3<-paste(fileName,window,thisTime,functionName,"Cos.rtf",sep="")
  if (Output$Txt){
    sink(fileName2, append=FALSE, split=TRUE)
  }
  if (!Console){   # if console is set to off, check the type of output for graphics
      if (Graphics == "pdf"){
        fileName4<-paste(fileName,window,thisTime,functionName,"Cos.pdf",sep="")
        pdf(file=fileName4,width=8, height=10)
      } else {if (Graphics == "jpg"){
                fileName4<-paste(fileName,window,thisTime,functionName,"Cos.jpg",sep="")
                jpeg(filename=fileName4,width=8, height=10)
                } else {if (Graphics == "png"){
                          fileName4<-paste(fileName,window,thisTime,functionName,"Cos.png",sep="")
                          png(filename=fileName4,width=8, height=10)
                        } else {if (Graphics == "tif"){
                          fileName4<-paste(fileName,window,thisTime,functionName,"Cos.tif",sep="")
                          tiff(filename=fileName4,width=8, height=10)
                        }
                }
                }
      }
  }
#####################################################################################################
#                                                                                                   #
#        Read data file;  assume tab delimited;  if that fails, read as CSV;  get header;           #
#                omit missing data and count missing data;                                          #
#                                                                                                   #
#####################################################################################################

  MyDataDelimiter <- read.table(fileName, nrows=30, header=header, sep="\t",stringsAsFactors=FALSE,skip=Skip)          
  #  only read a few rows to determine if CSV or tab delimited
  if (length(MyDataDelimiter[5,])>1){   #  if count >1, this is tab delimited
    delim<-'\t'
  } else {delim<-','}

  headerChar1<-''         # Use NA as a comment indicator -- not sure if this will work*****
  myHeader<-NULL
  if (delim=='\t'){  #  this is for one special kind of file output  AB
    if (header==TRUE){
      myHeader <- read.table(fileName, header=FALSE, nrows=1, sep="\t",skip=Skip)   # grab the header to extract character (tell it header is false)
      #get the header for names  -- all strings matching header will be removed
      headerChar1 <- substr(names(myHeader)[1],1,1)    # gets the 1st character of the header to be used as a comment indicator
    }  #  end if header=true  (nrows=0 makes sure 1st line is read  -- it was being skipped)
    MyDataReada <- read.table(fileName, header=header, nrows=0, sep="\t",stringsAsFactors=FALSE, skip=Skip,comment.char=headerChar1,col.names = names(myHeader))
  } else {    # end if delimiter is tab
        tableColTypes<-c("character","character","character","character","character","character","character")   # need this for FW
        
        tableColTypes[Y]<-"numeric"         #  works even when Y is a vector
        if (all(TimeCol>0) && all(TimeCol<=10)) {
            tableColTypes[TimeCol]<-"character"
        } #  else {  
        # if (any(TimeCol=1)){
#           message<-"ERROR:  The first column is reserved for character values, often used for IDs."
#           errMsg<-paste(errMsg,message)
#           closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
#           stop(message)
#        } else {
#           message<-"ERROR:  Time must be in the first 10 columns, for .CSV files."
#           errMsg<-paste(errMsg,message)
#           closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
#           stop(message)
#         }

     MyDataReada <- read.table(fileName, header=header, sep=',', stringsAsFactors=FALSE, skip=Skip,colClasses=tableColTypes,blank.lines.skip=TRUE,na.strings=c(" ","--"))
  } # end if delimiter is comma
  for (y in Y){
    MyDataRead<-subset(MyDataReada, !is.na(MyDataReada[,y]))       # omit data row if any Y column is NA
  }
  missingData<-(length(MyDataReada[,TimeCol])-length(MyDataRead[,TimeCol]))/length(MyDataReada[,TimeCol])    #  rowsData<-length(MyDataReada[,1])
 
  #MyDataRead<-subset(MyDataReada, !is.na(MyDataReada[,Y]))       # omit data row if Y column is NA
  
  #MyDataRead = na.omit(head(MyDataRead1,n=-2))   #  will drop the very last row in case it had no data
#####################################################################################################
#                                                                                                   #
#        Time column: convert time format to numeric if needed, or to number of Units of time       #
#                May be in one or two columns;  2 columns must be appended before conversion        #
#                                                                                                   #
#####################################################################################################

  if (timeFormat=="numeric"){
    #BaseTimeNum <- as.POSIXct(strptime(x=, format="%d/%m/%y %H:%M:%S", tz=tz))
    #StartDate <-paste(substr(MyDataRead[1,TimeCol],1,8),"0000",sep="") 
    #BaseTimeDay<-as.POSIXct(strptime(x=Sys.Date(), "%Y%m%d",tz=tz))     # midnight today
    #BaseTimeStr <- format(Sys.Date(),"%Y%m%d%H%M%S")
    #BaseTimeNum<-difftime(Sys.Date(),Sys.time(),units="secs")
    #NewBase<-difftime(BaseTimeNum,BaseTime)
    #MyDataRead[,TimeCol]<-format(BaseTime+BaseTimeNum + as.numeric(MyDataRead[,TimeCol])*3600,"%Y%m%d%H%M")   # convert to hours, and add to today's date.  (used to use BaseTime, but don't want time part)
    MyDataRead[,TimeCol]<-format(BaseTime + as.numeric(MyDataRead[,TimeCol])*3600,"%Y%m%d%H%M")   # convert to hours, and add to today's date.
    #RefDateTime<-MyDataRead[1,TimeCol]
  } else {   #convert time from 2 columns  (only one format allowed if time is in 2 columns:   17/10/07 \t 19:00:20)
    if (length(TimeCol)==2){
      MyDataRead[,TimeCol[1]] <- as.POSIXct(strptime(x=paste(MyDataRead[,TimeCol[1]],MyDataRead[,TimeCol[2]]), format="%d/%m/%y %H:%M:%S", tz=tz))
      MyDataRead[,TimeCol[2]] <- format(MyDataRead[,TimeCol[1]],"%Y%m%d%H%M")
      names(MyDataRead)[TimeCol[2]] <- "Datetime"
      TimeCol<-TimeCol[2]
    } else {   #convert time from 1 columns
      if (length(TimeCol)==1){
        MyDataRead[,TimeCol[1]] <- as.POSIXct(strptime(x=MyDataRead[,TimeCol[1]], format=timeFormat, tz=tz))
        MyDataRead[,TimeCol[1]] <- format(MyDataRead[,TimeCol[1]],"%Y%m%d%H%M")
        names(MyDataRead)[TimeCol[1]] <- "Datetime"
      }}}
#####################################################################################################
#                                                                                                   #
#        RangeDateTime$Start and $End: calculate total time length, StartDate and EndDate           #
#                May be in one or two columns;  2 columns must be appended before conversion        #
#                    NA- use the 1st time in the data set                                           #
#                    0- use Midnight at the start/end of the data                                   #
#                Calculate start and ending indexes in the data                                     #
#                print actual hours to be used for analysis, and 1st and last data point in file    #
#                                                                                                   #
##################################################################################################### 

  str(MyDataRead)                                         #  removed +3600 from *tz=tz))+3600)*       6/14
  AnalyzeLength<-(as.POSIXct(strptime(tail(MyDataRead[,TimeCol],1), "%Y%m%d%H%M",tz=tz))) - as.POSIXct(strptime(MyDataRead[1,TimeCol], "%Y%m%d%H%M",tz=tz)) 
  MyData_HRs<-(AnalyzeLength)*24  # hours in length of actual data

  if (is.na(StartDate)){
    StartDate <- format(as.POSIXct(strptime(MyDataRead[1,TimeCol], "%Y%m%d%H%M",tz=tz)),"%Y%m%d%H%M")
  } else {if (StartDate == 0){
    StartDate <-paste(substr(MyDataRead[1,TimeCol],1,8),"0000",sep="")                #  Use StartTime as start of analysis date-time
  }} #as.POSIXct(strptime(MyData[1,TimeCol], "%Y%m%d",tz=tz))
  MyData_length<-length(MyDataRead[,1])

  StartIndex<-which(MyDataRead[,TimeCol]>=StartDate)   #  where is the desired startTime?  (if start point is later than first data point)
  if (is.na(EndDate)){                                                     #  removed tz=tz))+3600      6/14
    EndDate <- format(as.POSIXct(strptime(tail(MyDataRead[,TimeCol],1), "%Y%m%d%H%M",tz=tz)),"%Y%m%d%H%M") # should these be pasted or posix?  todo
    } else {if (EndDate == 0){
    EndDate <-paste(substr(as.numeric(MyDataRead[MyData_length,TimeCol])+as.numeric(10000),1,8),"0000",sep="")                #  Use StartTime as start of analysis date-time
  } }

  EndIndex<-which(MyDataRead[,TimeCol]<=EndDate)   #  where is the desired endTime? 
  #cat("Check that your data file and End Date date are correct.  Data file begins:",MyDataRead[1,TimeCol],"; End Date is: ",EndDate,"\n")

  MyData<-MyDataRead[StartIndex[1]:tail(EndIndex,n=1),]     #  grab all the data from array after StartTime, before EndTime
  
  cat("There are ",MyData_HRs," actual hours of data in this file, ",MyDataRead[1,TimeCol]," to ",MyDataRead[tail(EndIndex,n=1),TimeCol],", and ",MyData_length,"data points.\n  %",missingData,"of data points are missing.\n")       #  Analysis will proceed from",StartDate," to ",EndDate,"\n")

  print(MyDataRead[1,])
  print(tail(MyDataRead,1))
#####################################################################################################
#                                                                                                   #
#        RefDateTime: get correct reference time;  subtract from data, start tim, and end time      #
#                convert time to a count in Units, as set by Units parameter                        #
#                    calculate MyData$time.n, MyData$time.hour and  MyData_length (used in analysis)#                                          #
#                    print total hours used in analysis and start/end times                         #
#                                                                                                   #
##################################################################################################### 

  # convert time
  MyData$time=as.POSIXct(strptime(MyData[,TimeCol], "%Y%m%d%H%M", tz=tz))     #   tz="GMT"
  # convert time to a number (time in seconds, converted into hours)
  # should probably be done with a hard coded time zone, so CDT and DST don't cause problems

  if (is.na(RefDateTime)){
    RefTime <-as.POSIXct(strptime(MyData[1,TimeCol], "%Y%m%d%H%M",tz=tz))                #  Use StartTime as reference date-time
    RefTimeString <- MyData[1,TimeCol]
  } else if (RefDateTime==0){
    RefTime<-as.POSIXct(strptime(MyData[1,TimeCol], "%Y%m%d",tz=tz))     # midnight of first day of data
    RefTimeString <- paste(substr(MyData[1,TimeCol],1,8),"0000",sep="")
     } else {RefTime<-as.POSIXct(strptime(RefDateTime, "%Y%m%d%H%M",tz=tz))
             RefTimeString <- RefDateTime}

  StartTime<-(as.numeric(as.POSIXct(strptime(StartDate, "%Y%m%d%H%M",tz=tz)))-as.numeric(RefTime))/3600      # convert to hours
  EndTime<-(as.numeric(as.POSIXct(strptime(EndDate, "%Y%m%d%H%M",tz=tz)))-as.numeric(RefTime))/3600   # convert to hours
  MyData_hours<-as.numeric(EndTime-StartTime)           #   get number of hours to analyze

  MyData$time.n=as.numeric(MyData$time) - as.numeric(RefTime)
  MyData$time.hour<-MyData$time.n/3600
  MyData_length<-length(MyData[,1])
  #MyData_hours<-(MyData$time.hour[MyData_length]-MyData$time.hour[1])  # hours in length of data if start date is after 1st data point
  # calculate number of hours in the full regression (Units*RegressionLen) -- it may not use all of the data
  # this is used for outer loop, full analysis
  #if (Units=='Day'){
  #  MyData_hours<-RegressionLen*24
  #} else if (Units=='Year'){
  #        MyData_hours<-24*365*RegressionLen
  #        } else if (Units=='Week'){
  #                MyData_hours<-24*7*RegressionLen    
  #              }
  cat("The program is using ",MyData_hours," hours for the analysis, starting at hour",StartTime,"and ending at hour",EndTime," relative to RefTime:",RefTimeString,"\n")
  # calculate number of hours in the Interval and Increment (Units*Interval, Units*Increment)
  # this is used for inner loop, progressive analysis of intervals over the full data set
  if (Interval==0 || Increment==0){     #  || oneCycle[1]>0){     #  No progression -- analyze full dataset
    Interval <-MyData_hours
    Increment<-MyData_hours
  } else if (Units=='hour'){
        Interval<-Interval
        Increment<-Increment
      } else if (Units=='day'){
            Interval<-Interval*24
            Increment<-Increment*24
          } else if (Units=='year'){
                  Interval<-Interval*24*365
                  Increment<-Increment*24*365
                } else if (Units=='week'){
                        Interval<-24*7*Interval      
                        Increment<-24*7*Increment
                      } else {
                        message<-paste("ERROR:  Units is improperly given: ",Units," Valid values:  hour, day, week, year")
                        errMsg<-paste(errMsg,message)
                        closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
                        stop(message) 
                      }
  if (Period$Start==0){       
    Period$Start <-Interval
  } else if (Period$Start<0){
            message<-"ERROR:  The parameter Period$Start cannot be negative.\n"
            errMsg<-paste(errMsg,message)
            closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
            stop(message)
          } else if (Period$Start>(1.25*MyData_hours)){        #  validation of Period$Start   
            message<-paste("ERROR:  The parameter Period$Start",Period$Start, "is much longer than the observation period",MyData_hours,".  Unreliable results.\n")
            errMsg<-paste(errMsg,message)
            print(message)
          } else if (Period$Start>MyData_hours){        #  validation of Period$Start   
            message<-paste("Warning:  The parameter Period$Start",Period$Start, "is slightly longer than the observation period",MyData_hours,".  Results may be unreliable.\n")
            errMsg<-paste(errMsg,message)
            print(message)
            } # end else Period$Start==0
  if (FreqInc==0){
    FreqInc<-1
    paramMsg<-paste("\n  TimeCol=",TimeCol,",  Y=",Y, ",  header=",header,"\n --  Periods=",Period["Set"],", Units=",Units, ",  Interval=",format(Interval,nsmall=3), ",  Increment=",format(Increment,nsmall=3), "\nPeriod$Start=",format(Period$Start,nsmall=3), ",  FreqInc=",format(FreqInc,nsmall=3), ",  Period$End=",format(Period$End,nsmall=3), "\nRefDateTime=",RefDateTime, ", StartTime=",format(StartTime,nsmall=3),", EndTime=",format(EndTime,nsmall=3),"Percent of missing (blank) sample values: %",missingData,"\n",functionName,"\n")
  } else if (FreqInc<0){
      message<-"ERROR:  The parameter Period$Increment cannot be negative.\n"
      errMsg<-paste(errMsg,message)
      closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
      stop(message)
    } 

  if (Period$End==0){
    if (dt==0){
      Period$End<-4
    } else if (Period$End<0){
      message<-"ERROR:  The parameter Period$End cannot be negative.\n"
      errMsg<-paste(errMsg,message)
      closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
      stop(message)
    } else if (Period$End>MyData_hours){
      message<-"ERROR:  The parameter Period$End is larger than the number of hours.  ???\n"
      errMsg<-paste(errMsg,message)
      print(message)
    } else Period$End<-2*dt
    if (Period$End>Period$Start){
      Period$End<-Period$Start/4
      FreqInc<-.1
    }
  }
  cat(Period$Start," Period$Start ",Period$End," Period$End ",Period$Start,"Period$Start",Interval,"Interval",Increment,"Increment\n")

paramMsg<-paste("\n  TimeCol=",TimeCol,",  Y=",Y, ",  header=",header,"\n --  Periods=",Period["Set"],", Units=",Units, ",  Interval=",format(Interval,nsmall=3), ",  Increment=",format(Increment,nsmall=3), "\nPeriod$Start=",format(Period$Start,nsmall=3), ",  FreqInc=",format(FreqInc,nsmall=3), ",  Period$End=",format(Period$End,nsmall=3), "\nRefDateTime=",RefDateTime, ", StartTime=",format(StartTime,nsmall=3),", EndTime=",format(EndTime,nsmall=3),"Percent of missing (blank) sample values: %",missingData,"\n",functionName,"\n")


  #Variable<-paste(fileName1,":V",Ycol,sep="")
  par(mar=c(4,4,1,1)) # set up margins
  
  #  #####    note on indexes and variables:  one set tracks through the ACTUAL times and data points:  #####
  #                EndIdx, StartIdx, thisIdxCnt, MyData_length, TimeIdx
  #  #####    another set tracks through the periods, in hours, being calculated:       ######
  #                MyData_hours; StartSpans, Interval, Increment, Progression_end, StartTime, StartSpanTime, EndTime
  #  #####    These two must, of course, align. See:   TimeIdx<-which(StartSpanTime <= MyData$time.hour) 
  EndIdx<-0

if (Interval > MyData_hours){    #  warn if interval chosen is longer than data
  message<-paste("Warning: Chosen Progressive$Interval (",Interval,") is longer than data file (",MyData_hours,") (MyData_hours).\n")
  print(message)
  errMsg<-paste(errMsg,message)
} 
if (Increment> MyData_hours) {
  message<-paste("Warning:  Chosen Progressive$Increment (",Increment,") is longer than data file (",MyData_hours,") (MyData_hours).\n")
  print(message)
  errMsg=paste(errMsg,message)
}
  # seq(72-84+1,192-50.4,by=8.4)
  if (MyData_hours==Interval){    #StartTime==MyData$time.hour[1] && 
    StartSpans<-seq(from=0,to=MyData_hours-1,by=Increment)        #StartSpans<-seq(1,Interval, by=Increment)
  #} else {StartSpans<-seq(from=(StartTime),to=(StartTime+MyData_hours-Interval),by=Increment)  
  } else {if ((MyData_hours-Interval+Increment)<=0){
              errMsg<-paste(errMsg,"Error:  MyData_hours-Progressive$Interval+Progressive$Increment is < 1:  ",MyData_hours,"-",Interval,"+",Increment,".  Using ",MyData_hours,"as Interval.\n")
              closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
              Interval<-MyData_hours
              }
      StartSpans<-seq(from=0,to=(MyData_hours-Interval+Increment),by=Increment)     #  one too many (SS), perfect for all (NA/NA)
          #  to=(MyData_hours-1-Interval):  added + Increment because 3varsProg was doing 1 too few spans
          #  to=(MyData_hours-1-Interval+Increment):  #best  2 too few when inc=.5 interval=24  serial section
          #  to=(MyData_hours+1-Increment)         7 too many when inc=.5 interval=24 serial section
          #  to=(MyData_hours-Interval+Increment)  one too many  (should be LESS THAN last span value) when inc=.5 interval=24  serial section
          }   #StartSpans<-seq(1,Interval, by=Increment)
  #print(StartSpans)

  Progression_end<-length(StartSpans)
  #  print Y title, and points Interval
  sumN <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumLow <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumHi <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumMean <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumMedian <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumMode <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumSD <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  sumT <- matrix(data=NA,nrow=Ys_end,ncol=Progression_end+1)
  #sumN[Progression_end+1]<-MyData_length
  yNew<-0

  for (y in Y){   #  parse each Y columng to get the summary component of each;  goes in final row after all j summaries
    yNew<-yNew+1
    sumN[yNew,Progression_end+1]<-MyData_hours
    sumLow[yNew,Progression_end+1]<-min(MyData[,y], na.rm=TRUE)
    sumHi[yNew,Progression_end+1]<-max(MyData[,y], na.rm=TRUE)
    sumMean[yNew,Progression_end+1]<-mean(MyData[,y], na.rm=TRUE)
    sumMedian[yNew,Progression_end+1]<-median(MyData[,y], na.rm=TRUE)   #  could be proper dt if VERY irregular data
    sumTab<-tabulate(MyData[,y])
    sumSD[yNew,Progression_end+1]<-sqrt(var(MyData[,y],y=NULL))
    dTtest<-which(sumTab == max(sumTab, na.rm=TRUE))   #  way to find dt for non-equidistant data
    sumMode[yNew,Progression_end+1]<-NA
    if (length(dTtest)>4){    #use mean instead of median
      sumT[yNew,Progression_end+1] <- MyData_hours + sumMean[yNew,Progression_end+1]
    } else if  (length(dTtest)>1){    # take the mean of the 2 or 3 max values
      sumT[yNew,Progression_end+1] <- MyData_hours + mean(dTtest, na.rm=TRUE)
      } else {sumMode[yNew,Progression_end+1]<-which(sumTab == max(sumTab, na.rm=TRUE))   #  way to find dt for non-equidistant data
              sumT[yNew,Progression_end+1] <- MyData_hours + sumMode[yNew,Progression_end+1]
              }
    #   sumT      hours+dt=T acknowledges that most times points are binned, so should be more than actual hours in file
  }
  
  Page<-0
  if (oneCycle[1]==0){                    #  if oneCycle != 0, Period$End and FreqInc are not used.
    if (Period$End <= 0 || FreqInc <= 0){
      message<-paste("ERROR:  Period$End (",Period$End,") and Period$Increment (",FreqInc,") cannot be 0 when Period$Set is 0.\n")
      errMsg<-paste(errMsg,message)
      closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
      stop(message)
    }
    if (Period$Start<Period$End){
      message<-paste("ERROR:  The parameter Period$Start (",Period$Start,") cannot be smaller than Period$End (",Period$End,") when Period$Set is 0.\n")
      errMsg<-paste(errMsg,message)
      closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
      stop(message)
    }
    if (FreqInc > Period$Start){    #  is this needed??    6/14
      message<-paste("ERROR:  Period$Increment (",FreqInc,") cannot be larger than Period$Start (",Period$Start,") (or Progressive$Interval) when Period$Set is 0.\n")
      errMsg<-paste(errMsg,message)
      closeOutput(file=fileName4,output=Output,console=Console,opar=opar,ERR=TRUE,errMsg=errMsg,paramMsg=paramMsg)
      stop(message)
    } else if (FreqInc>1) {
      message<-"Warning:  Chosen Increment (>1) is less than optimal.  Increments would optimally be <=1.\n"
      print(message)
      #closeOutput(file=fileName4,output=Output,console=Console,opar=opar)
      errMsg=paste(errMsg,message)
    }
    
    #RowCnt<-((Period$Start/Period$End)-FreqInc)/FreqInc    #  with GCG assistance -- should work for all
    #RowCnt<-((Period$Start/Period$End)/FreqInc)          #   (1/Period$End-1/Period$Start)/FreqInc is .1 too low for inc of .001
        # may need to add 1 if freqinc>1, and add FreqInc if FreqInc<1   
        #((1/Period$End)/(1/Period$Start)-FreqInc)/FreqInc   no   (Period$Start-Period$End)/FreqInc     
#     if (FreqInc<1){
#       RowCnt<-floor(((Period$Start/Period$End)-FreqInc)/FreqInc)
#     } else {
#       RowCnt<-ceiling(((Period$Start/Period$End)/FreqInc)) #+1     #   (Period$Start-Period$End)/FreqInc  #  (Interval/2)/FreqInc
#     }
    # the integer (truncated) part of 1 + (tau-s/tau-e -1)/delta.
    RowCnt<-floor(1 + ((Period$Start/Period$End)-1)/FreqInc)      #  using ceiling with out 1+ is not equivalent, as non integer result is 1 too small
    RowCntAlt2<-floor(1 + ((1/Period$End)/(1/Period$Start)-1)/FreqInc)    #  for example:  start=52, end=2, inc = 5 should give 6 results;  (52/2)+1 harmonics
    
    if (RowCnt<1){
      RowCnt<-1                              #  8/11/14  for LS needs to be 1   (was 2)
    }
  } else {if (Components>1){
              RowCnt<-1}
          else {RowCnt <- length(oneCycle)}   #  if oneCycle != 0, reset Period$End and FreqInc to default=1
          #Period$Start <-Interval
          #FreqInc <-1     #  6/9/14 Period$Start and FreqInc may be set from Period$Start and Period$Inc, so cannot always be reset to default.  Is there a time they should be reset?
          print("resetting freq start and inc")
  }
  
  M <- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  newPR <- array(data=NA,c(RowCnt, Progression_end,Components))
  PHI <- array(data=NA,c(RowCnt, Progression_end,Components))
  PHIr <- array(data=NA,c(RowCnt, Progression_end,Components))
  A <- array(data=NA,c(RowCnt, Progression_end,Components))
  PR <- array(data=NA,c(RowCnt, Progression_end,Components))
  P <- array(data=NA,c(RowCnt, Progression_end,Components))
  testPg <- array(data=NA,c(RowCnt, Progression_end,Components))
  testPb <- array(data=NA,c(RowCnt, Progression_end,Components))
  F <- array(data=NA,c(RowCnt, Progression_end,Components))
  mesor_se <- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  phi_se <- array(data=NA,c(RowCnt, Progression_end,Components))
  amp_se <- array(data=NA,c(RowCnt, Progression_end,Components))
  Cycle <- array(data=NA,c(RowCnt, Progression_end,Components))
  Err <- array(data="",c(RowCnt, Progression_end,Components))
  hours <-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  nPts<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  sPts<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  time<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  yVar<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  MSS<- vector(mode="integer",length = Components)
  if (Components>1){
    multi_P<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
    multi_PR<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
    Magnitude<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
    Orthophase<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
    Bathyphase<-matrix(data=NA,nrow=RowCnt, ncol=Progression_end)
  }
  newData<- matrix(data=NA,nrow=RowCnt, ncol=Progression_end)    # holds filtered or unfiltered data, myData for each Interval

  for (y in 1:Ys_end){
    Ycol<-Y[y]
    paramMsg<-paste("\n  TimeCol=",TimeCol,",  Y=",Y, ",  header=",header,"\n --  Periods=",Period["Set"],", Units=",Units, ",  Interval=",format(Interval,nsmall=3), ",  Increment=",format(Increment,nsmall=3), "\nPeriod$Start=",format(Period$Start,nsmall=3), ",  FreqInc=",format(FreqInc,nsmall=3), ",  Period$End=",format(Period$End,nsmall=3), "\nRefDateTime=",RefDateTime, ", StartTime=",format(StartTime,nsmall=3),", EndTime=",format(EndTime,nsmall=3),"Percent of missing (blank) sample values: %",missingData,"\n",functionName,"\n")
    # Use the periodogram
    if (window!="noTaper"){
      CatWindowList<-CatWindow(window=window,myData=MyData[,Ycol],Time=MyData$time.hour,dataN=MyData_hours,Start=MyData$time.hour[1]+StartSpans[1]-1)    # plots the full set of data w/filter
    }
    layout(matrix(c(1,2), 2, 1, byrow = TRUE), 
          widths=c(1), heights=c(1,2))          #   c(figure#1, figure#2), 2 row, 1 col, ...
    par(mar=c(4,4,2,1), oma=c(2,0,1,0))         # set up margins
    if (dt!=0 && window!="noTaper"){
      CatPeriodogram<- CatPeri(CatWindowList$data,binsPerHr=1/dt, export=T, fileoutP=paste(fileName,thisTime,functionName,"V",Ycol,"CosPgram.txt"), Hx=.15, cex=.9, Debug=FALSE)    # pass samples/day when units=hrs
      View(CatWindowList$data)
    } else {plot.new()}

    n<-length(MyData[,Ycol])
    if (is.numeric(MyData[n, TimeCol])){
      startDateP<-as.POSIXct(strptime(x=MyData[1,TimeCol], format="%Y%m%d%H%M", tz=tz))
      endDateP<-as.POSIXct(strptime(x=MyData[n,TimeCol], format="%Y%m%d%H%M", tz=tz))
    } else {startDateP<-MyData[1,TimeCol]
            endDateP<-MyData[n,TimeCol]}
    title(paste('Start time:',startDateP,';    End time:',endDateP),cex=.7)
    #-------------------
#     if (Plex==TRUE){
#       CatPlexigram<- CatPlex(CatWindowList$data, foldLen=foldLen, Debug=FALSE) 
#     }

      cat("\n\tPR\t\t   F\t\t\tP \tSS[j]\t i\t cycle \t  Mesor \t  s.e. \t \t  Amp \t\t s.e. \t \tPhi\t\t s.e.\n")
      #cat("-------------------------------------------------------\n")   
      for (j in 1:Progression_end){   #  make a vector with incremental values.  use var(i) for necessary indexes
      # from:to
        if (j %in% c(1,10)){
          if (j!=1){
            par(mfrow=c(1,1),mar=c(4,4,4,1))
            jht<-2.5
            cexVar<-.5
            htVar<-.3
          } else {jht<-0
              cexVar<-.7
              htVar<-.8}
          Page<-Page+1
          if (oneCycle[1]>0){
            main<-paste("Each Period: Var",y)
            } else {main<-paste("COSINOR Maximum Period: Var",y)}    
          
          plot(0:10, 0:10, type="n",xlab="",ylab="",axes=FALSE, main=main,cex=1)
          text(c(-.2,1.5,2.7,3.5,4.3,5.3,6.3,7.3,8.3,9.3),c(9.5,9.5,9.5,9.5,9.5,9.5,9.5),labels=c("Interval","Period","P","PR","Mesor", "s.e.","Amp","s.e.","Phi","s.e."),adj = c(0,0))   #  adj = c(0,0) left justifies
          abline(h=9)
          #page footer
          mtext(paste('Function:',functionName,"  ",fileName1,"; TimeCol=",TimeCol," Ycol=",Ycol, " RefDateTime=",RefDateTime),side=1,line = 1, cex =.5, adj=.9)
          mtext(paste("P",Page,"  ",window,"  ",thisTime," Units=",Units, " Interval=",Interval, " Increment=",Increment, " header=",header," Set=",Period["Set"]),side=1,line = 2, cex =.5, adj=.9)
        }
        
#         if (j<7 & j>1){
#           browser()
#           print(EndIdx)
#           print(StartIdx)
#         } else 
          maxAmp = 1  
          StartSpanTime<-StartTime+StartSpans[j]    # Feb 2014 added back in; 11/4  StartTime+StartSpans[j]-1 # 10/18 added StartTime+ DH;  9/30  MyData$time.hour[1]+StartSpans[j]-1
          EndSpanTime<-StartSpanTime +Interval          # 6/14 no subtraction; 5/2014 subtract 1 oneSec instead; 9/30  -1***adj to match gc  Add Interval length to the last Interval StartSpans[j]+Interval to get the ending of the Interval
         # when using 24 hour progressive Interval, if should not go from 8:00 to 8:00 in one Interval, should only go from 8:00 to 7:59:59!  
        TimeIdx<-which(StartSpanTime <= MyData$time.hour)   #   get indices past the start of this Interval
        if (is.na(TimeIdx[1])){
          StartIdx<-length(MyData$time.hour)     #  use the largest index (last data point)
        } else StartIdx<-TimeIdx[1]                # index of starting time is FIRST index in array of indices
           if (EndSpanTime>MyData_hours+StartTime){   # Feb 2014 had to add + StartTime to these 2 lines
            EndSpanTime<-MyData_hours+StartTime
          }
          #if (EndSpan>EndTime){
          #  print("**************what to do about this???****************")
          #  EndSpan<-MyData_hours
          #}
          if (Progression_end==j){                  #  6/14   last (or only) spans need to be treated differently than all others
            # needed to round because equal numbers didn't compare as equal due to differing precisions  (116.5333333 <> 116.5333)
            TimeIdx<-which(round(EndSpanTime,digits=10) >= round(MyData$time.hour,digits=10))   #  should be gte;  get indices UP TO the end of this Interval
            #  in what case does this need to be gte?   ActivityP1-ZCM-SSend2 serial section might need it to be equal
          } else TimeIdx<-which(EndSpanTime > MyData$time.hour)    #  6/14
          EndIdx<-which.max(TimeIdx)               # index of ending time is LAST index in array if indices

          thisIdxCnt<-EndIdx-StartIdx+1   # data points in this Interval  -- used for S11, and SigmaHat
          Loops<-RowCnt          
          if (Components==1 && oneCycle[1] > 0){              #  a specified period in oneCycle overrides spectrum calculation  (and Period$Start/End?)
            Cycles<-oneCycle
            Loops<-length(Cycles)        
          } else if (Components>1){
            Cycles<-oneCycle
            Loops<-1
          }
          #cat("StSpan ",StartSpans[j]," Interval ",Interval," EndSpan ",EndSpan," StrtIdx ",StartIdx, " EndIdx ",EndIdx," thisIdxCnt ",thisIdxCnt," j ",j,"\n")

        if (window != "noTaper"){          # calculates a window for the Interval
          CatWindowList<-CatWindow(window=window,myData=MyData[StartIdx:EndIdx,Ycol],Time=MyData$time.hour[StartIdx:EndIdx],dataN=Interval,Start=StartSpanTime)
          newData<-CatWindowList$data
        }
        else {newData<-MyData[StartIdx:EndIdx,Ycol]}
        #  summary is on the filtered data.
        sumN[y,j]<-length(newData)  # should be changed to this
        #sumN<-length(newData[,Ycol])
        sumLow[y,j]<-min(newData, na.rm=TRUE)
        sumHi[y,j]<-max(newData, na.rm=TRUE)
        sumMean[y,j]<-mean(newData, na.rm=TRUE)
        sumMedian[y,j]<-median(newData, na.rm=TRUE)   #  could be proper dt if VERY irregular data
        sumTab<-tabulate(newData)

        sumSD[y,j]<-sqrt(var(newData,y=NULL))
        dTtest<-which(sumTab == max(sumTab, na.rm=TRUE))   #  way to find dt for non-equidistant data
        sumMode[yNew,Progression_end]<-NA
        if (length(dTtest)>4){    #use mean instead of median
          sumT[y,j] <- MyData_hours + sumMean[y,j]
        } else if  (length(dTtest)>1){    # take the mean of the 2 or 3 max values
          sumT[y,j] <- MyData_hours + mean(dTtest, na.rm=TRUE)
        } else {sumMode[y,j]<-which(sumTab == max(sumTab, na.rm=TRUE))   #  way to find dt for non-equidistant data
                sumT[y,j] <- MyData_hours + sumMode[y,j]  # acknowledges that most times points are binned, so should be more than actual hours in file
        }

          for (i in 1:Loops){
            # fit sinusoidal model
            if (oneCycle[1]==0){                  #  if a period has been specified, use present cycle
              #cycle<-Interval/(i-(StartSpans[j]-1))       #  divisor should start at one each time this loop restarts
              cycle<-Period$Start/(1+((i-1)*FreqInc))          ####  Interval/(1+((i-1)*FreqInc))  from Interval/i
              } else cycle<-Cycles[i]

            Cycle[i,j,1]<-cycle                    # store for later 
            yVar[i,j]<-Y[y]
            hours[i,j] <- paste(format(StartSpanTime,nsmall=1)," - ",format(EndSpanTime,nsmall=1),sep="")  # took :59 off because can be decimal hrs;  this shows ALL used
            cycleCnt<-(EndSpanTime-StartSpanTime)/cycle
            minPtCnt<-2*cycleCnt+1
            nPts[i,j] <- EndIdx-StartIdx+1
            sPts[i,j] <- paste(StartIdx,"-",EndIdx,"\n(",nPts[i,j],")")
            time[i,j] <- paste(MyData[StartIdx,TimeCol],"-\n",MyData[EndIdx,TimeCol])
            if (nPts[i,j]<=minPtCnt) {        #  not enough data points for the trial period being analzyed.
              print(paste(keyMsgs[5],"C=",2*cycleCnt+1,"min pts=",minPtCnt))
              Err[i,j,]<-paste(Err[i,j,],errKey[5],"min pts=",minPtCnt)   # "@"
              #next   should not skip calculations  -- calculate as many as possible  jiii
            }
            if (cycle>(MyData$time.hour[EndIdx]-MyData$time.hour[StartIdx])) {     #  data covers 90% of timeperiod < target period
              if ((.9*cycle)>(MyData$time.hour[EndIdx]-MyData$time.hour[StartIdx])) {    #  Error if much smaller than cycle
                print(keyMsgs[7])
                Err[i,j,]<-paste(Err[i,j,],errKey[7])   # "@@@"
                #next
              } else {  #  Warning if cycle about same size as data Interval
                print(keyMsgs[8])
                Err[i,j,]<-paste(Err[i,j,],errKey[8])   # "@@"
              }
            }

#             if (!any(diff(MyData$time.hour[StartIdx:EndIdx],lag=(2*Components+1))<=cycle)){
#               message<-"Warning:  There should be more than 2p+1 data points in an interval the length of the target period being assessed, according to Nyquist (p=#Components).  Results may be unreliable.\n"
#               print(message)
#               Err[i,j,]="@@"
#             }

            # still need to check for gaps
            #Err[i,j,]<-" "   doesn't need to be set because it defaults to NA

            CosMatrix<-matrix(data=NA,nrow=CosMatrixDim, ncol=CosMatrixDim)
            testXmean<-matrix(data=NA,nrow=CosMatrixDim, ncol=Components)
            YMatrix<-matrix(data=NA,nrow=CosMatrixDim, ncol=1)
            CosCoMatrix1<-matrix(data=NA,nrow=CosMatrixDim, ncol=thisIdxCnt)
            CosCoMatrix1[1,]<-rep(x=1,times=thisIdxCnt)         # need to be 0s for CosMatrix<-Sum...

            for (m in 1:Components){
              
              #Err[i,j,m]<-" "
              if (Components>1){    #  if components >1, override value of cycle set above.
                cycle<-Cycles[m]
                Cycle[i,j,m]<-cycle
                # calculate dTime for use in magnitude:  if all components are multiples of largest, divide one period into 360 (outside loop)
                #       otherwise, divide full length of data hours into 360;  -->dT     Starttime+dT-->T
                if (LCM<=Interval){
                  dTime<-LCM/360        # length of time units for each degree of one period
                } else {
                  dTime<-Interval/360
                }
                magTime<-vector(mode="integer",length = 360)
                # relative to RefTime, so start at 0
                magDt<-(360*dTime)              #  this is the #hours from RefTime of the last time
                magTime<-seq(from=0,by=dTime,to=magDt-dTime)
                #View(magTime)
                #magTime<-seq(from=MyData$time.hour[StartIdx],by=dTime,to=(MyData$time.hour[StartIdx]+(359*dTime)))
                #magTimeEnd<-which(MyData$time.hour>=(360*dTime))[1]            #  this is the index of the last time
                magTimeEnd<-RefTime + (magDt*3600)               #  this is the last time, in time format (convert hrs to sec)
                
              }
              CosCoMatrix1[m*2,]<-cos(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)     # x1 for  cycle m
              CosCoMatrix1[m*2+1,]<-sin(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)   # z1 for  cycle m
              YMatrix[m*2] <- sum(newData%*%(cos(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)))     # Y*x1
              YMatrix[m*2+1] <- sum(newData%*%(sin(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)))   # Y*z1 
            }

            # vector multiplication to build matrix for cosine
            for (m in 1:CosMatrixDim){      #  columns
              for (o in 1:CosMatrixDim){    #  rows
                  CosMatrix[o,m]<-sum(CosCoMatrix1[m,] * CosCoMatrix1[o,])
              }
            }
            CosMatrix[1,1]<-thisIdxCnt
            YMatrix[1] <- sum(newData)
                      
            mdat<-CosMatrix
            mdat2<-YMatrix
            #invert matrix to get the solution
            if (any(is.na(mdat))) {
              Err[i,j,]<-paste(Err[i,j,],errKey[1])         # "*"     #  Err holds all error symbols for this element;  add this error
              print(keyMsgs[1])
              next
            } else {
              if (det(mdat)<.0000001){
                Err[i,j,]<-paste(Err[i,j,],errKey[2])         # "**     #  Err holds all error symbols for this element;  add this error
                print(keyMsgs[2])
                next
              }  else {mdatInv <- solve(mdat)

            #multiply the inverted matrix by the dependent variable matrix to get vector M, B, G, B2, G2, ...   (x=inv(S)b)
            coefs <- mdatInv  %*%  mdat2
            #browser()
            if (Debug==TRUE || i==Interval){
                #print(TimeIdx)
              #print(crsprd)
              #print(s)
              #print(model$coefficients)
              #print(Amp)
              #cat("data22 ",MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle," cycle ",cycle,"\n")
              #cat("cos22 ",cos(MyData$time.hour[StartIdx:EndIdx]*(2*pi)/cycle)," cycle ",cycle,"\n")
              #cat("StartIdx ",StartIdx,"  EndIdx ",EndIdx," thisIdxCnt ",thisIdxCnt,"\n")
                print(mdat)
                print(mdat2)
                print(mdatInv)
                cat("matrix coefs ",coefs,"\n")
                #browser()         #########   Great Debug location
            }
            #calculate MESOR only once, regardless of # of components
            M[i,j]<-coefs[1]
            #calculate the Mean of Ys
            MeanR<-mean(newData, na.rm=TRUE)
            multi_Model_mag<-multi_Model_Y<-Model_Y <- M[i,j]          # Model_Y = Mesor +  bX + gZ 
            
            for (m in 1:Components){
              beta<-m*2
              gamma<-m*2+1
              if (Components>1){    #  if components >1, override value of cycle set above.
                cycle<-Cycles[m]
                Cycle[i,j,m]<-cycle
              }
                #sometimes coefs will be undefined.  skip calculations and move to next cycle
                if (!is.na(coefs[beta]) && !is.na(coefs[gamma])){
                    #Calculate Amplitude using B and G   (coefs[beta]=Beta)
                    A[i,j,m]<-(coefs[beta]^2+coefs[gamma]^2)^0.5
                    #  cat("A",A[i,j,m],"max",A[maxAmp,j,m],"m",m,"maxAmp",maxAmp)
                    #preserve the index for the cycle with the maximum amplitude           
                    if (is.na(A[maxAmp,j,m])){
                      maxAmp<-i
                      } else if (A[i,j,m]>A[maxAmp,j,m]) {
                      maxAmp<-i
                    }   #  this does not capture the maxAmp for multiple components properly

                      # Calculate Model:  Y(t) = M + bX + gZ    where X=cos(t2pi/cycle) and Z = sin(t2pi/cycle)
                      Model_Y = M[i,j] + (coefs[beta]*cos(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle)) + (coefs[gamma]*sin(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle))
                      if (Components>1){      # multiple components model 
                        multi_Model_Y = multi_Model_Y + (coefs[beta]*cos(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle)) + (coefs[gamma]*sin(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle))  

                        #Calculate model of 1 period for magnitude:  Y(t) = M + bX + gZ + e(t)where  b = Acosf  and  g = -Asinfand  X = cos(2pt/t)  and Z = sin(2pt/t)
                        multi_Model_mag = multi_Model_mag + coefs[beta]*cos(2*pi*magTime/Cycles[m]) + coefs[gamma]*sin(2*pi*magTime/Cycles[m])
                      }

                     MYhat_Ymean<-Model_Y-MeanR     #   calculates MSS for single component model
                     MSS[m]<-sum(MYhat_Ymean^2)        #   calculates MSS for single component model
                      #  new PR 
                    yTSS<-newData-MeanR       #  vector Y-Ybar for each Y
                    TSS_2<-sum((yTSS)^2)       #   sum of Y-Ybar^2
                    #beta1   *  sum[(Y-Ybar)x1]  * gamma1  *  sum[(Y-Ybar)z1] /TSS^2
                    #beta2   *  sum[(Y-Ybar)x2]  * gamma2  *  sum[(Y-Ybar)z2] /TSS^2  (one for each component)
                     newPR[i,j,m]<-100*(coefs[beta]*sum(yTSS*cos(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle)) + coefs[gamma]*sum(yTSS*sin(2*pi*MyData$time.hour[StartIdx:EndIdx]/cycle)))/TSS_2        #  my Y-Ymean*x1*coef
                    if (newPR[i,j,m]<0){
                      Err[i,j,m]<-paste(Err[i,j,m],errKey[3])          #  +
                      print(keyMsgs[3])
                      #next            should do all calculations as possible;  multiple component PR doesn't get put into PR
                    }
                    #calculate phi:  atan(-G/B) + Kpi
                    ph<-atan(-coefs[gamma]/coefs[beta])
                    if (coefs[beta]>=0) {phi2<-ph}
                    if (coefs[beta]<0&coefs[gamma]>=0) {phi2<-ph+pi}
                    if (coefs[beta]<0&coefs[gamma]<0) {phi2<-ph-pi}
                    # put in 0 to 2pi range
                    PHIr[i,j,m]<-phi2
                    if (phi2<0) { PHIr[i,j,m]<-phi2+(2*pi)} 
                    if (phi2>(2*pi)) { PHIr[i,j,m]<-phi2-(2*pi)}
                    # convert to degrees betwen 0 and 360
                    PHI[i,j,m] <- (PHIr[i,j,m]/(2*pi))*360
                    if (PHI[i,j,m]>0) {PHI[i,j,m]<-PHI[i,j,m]-360}
                    
                } else {  #  end only-if coefs can be calculated  -- this error not needed
                        Err[i,j,m]<-paste(Err[i,j,m],errKey[4])   #  ++
                        print(keyMsgs[4])
                        #browser()
                        next
                }    #  end only-if coefs can be calculated
            }   # end coef calculation for each component

            # sum(Y-meanY)^2 = sum(Y - Yhat)^2 + sum(Yhat - meanY)^2
            if (Components>1){      # multiple components model 
              Model_Y <- multi_Model_Y       #  use the full model for calculations from here on
              # calculate MSS for the full model
              MYhat_Ymean<-Model_Y-MeanR
              multi_MSS<-sum(MYhat_Ymean^2)
            }
            MY_Yhat <- newData-Model_Y     # my Y-Yhat
            #browser()
            RSS<-sum(MY_Yhat^2)
            if (RSS==0) {
              print(keyMsgs[6])
              Err[i,j,]<-paste(Err[i,j,],errKey[6])         # "@@"
              next
            }
            df1<-Components*2
            df2<-thisIdxCnt-CosMatrixDim                      # N-2*Components+1

            if (Components>1){      # multiple components model 
              multi_F<-(multi_MSS/df1)/(RSS/(df2))       #  df1 = 2, since MSS is for one component;  df2 is for full model
              multi_P[i,j]<-1-pf(multi_F,df1=df1, df2=df2)              # Fisher–Snedecor (F ) (χ2 ) 
              multi_PR[i,j]<-(multi_MSS/(RSS+multi_MSS))*100
            } 
           
            SigmaHat<-(RSS/(df2))^.5
            #cat(thisIdxCnt," sHat ",SigmaHat,"\t",PR[i,j],"\t",F[i,j],"\t ",P[i,j],"\t ")

            mesor_se[i,j] <-SigmaHat * mdatInv[1,1]^.5

            #   for testing alternate formula for individual P
            for (m in 1:Components){
              F[i,j,m]<-(MSS[m]/2)/(RSS/(df2))       #  df1 = 2, since MSS is for one component;  df2 is for full model
              P[i,j,m]<-1-pf(F[i,j,m],df1=2, df2=df2)              # Fisher–Snedecor (F ) (χ2 )

              # testing alternate formula for individual P
              tInd1<-2*m
              tInd2<-2*m+1
              beta<-2*m
              gamma<-2*m+1
              #   excludes diagonals of inverse matrix, which may be important
              test<-(mdatInv[tInd2,tInd2]*coefs[beta]^2)+2*mdatInv[tInd1,tInd2]*coefs[beta]*coefs[gamma]+mdatInv[tInd1,tInd1]*coefs[gamma]^2
              test2<-test/(mdatInv[tInd1,tInd1]*mdatInv[tInd2,tInd2]-mdatInv[tInd1,tInd2]^2)
              test3<-test2/(2*SigmaHat^2)
              testPg[i,j,m]<-1-pf(test3,df1=2, df2=df2) 

              #  alternative calculation for F    p 409 Bingham [33] 
              testXmean<-(CosMatrix[1,tInd1])/thisIdxCnt    #  X mean
              testZmean<-(CosMatrix[1,tInd2])/thisIdxCnt    #  Z mean
              testX2<-sum((CosCoMatrix1[tInd1,] * CosCoMatrix1[1,]-testXmean)^2)   #  X 
              testZ2<-sum((CosCoMatrix1[tInd2,] * CosCoMatrix1[1,]-testZmean)^2)   #  Z 
              testXZ<-sum((CosCoMatrix1[tInd1,] * CosCoMatrix1[1,]-testXmean)*(CosCoMatrix1[tInd2,] * CosCoMatrix1[1,]-testZmean))   #  XZ 
              testF<-(testX2 * coefs[beta]^2 + 2*testXZ * coefs[beta] * coefs[gamma] + testZ2 * coefs[gamma]^2)/(2*SigmaHat^2)
              testPb[i,j,m]<-1-pf(testF,df1=2, df2=df2)
#browser()
              if (Components>1){      # multiple components model 
                PR[i,j,m]<-(MSS[m]/(RSS+multi_MSS))*100
                } else {
                PR[i,j,m]<-(MSS[m]/(RSS+MSS[m]))*100
              }
              
              #calculate standard error for mesor, amplitude, phi -- convert se for phi to degrees
              amp_se[i,j,m] <-SigmaHat * (mdatInv[beta,beta] * cos(PHI[i,j,m])^2 - (2 * mdatInv[beta,gamma] * cos(PHI[i,j,m]) * sin(PHI[i,j,m])) + mdatInv[gamma,gamma] * sin(PHI[i,j,m])^2)^.5
              phi_se[i,j,m] <- (SigmaHat * (mdatInv[beta,beta] * sin(PHI[i,j,m])^2 + (2 * mdatInv[beta,gamma] * cos(PHI[i,j,m]) * sin(PHI[i,j,m])) + mdatInv[gamma,gamma] * cos(PHI[i,j,m])^2)^.5)/A[i,j,m]
              phi_se[i,j,m]<-phi_se[i,j,m]*180/pi       # Convert to degrees
              if (Debug==TRUE){
                cat(" seM ",mesor_se[i,j]," seA ",amp_se[i,j,m]," sePHI ",phi_se[i,j,m]," \n")
                cat("test3",m,"   ",test3,"\n")
                cat("testPg",m,"   ",testPg[i,j,m],"\n")
                cat("testF",m,"   ",testF,"\n")
                cat("testPb",m,"   ",testPb[i,j,m],"\n")
              }
              cat(thisIdxCnt,"\t","sigma ",SigmaHat,"\t",PR[i,j,m],"\t",F[i,j,m],"\t ",P[i,j,m],"\t \n")
              cat(StartSpans[j],"\t",i,"\t",format(Cycle[i,j,m],width=3), "\t ",M[i,j],"\t", format(mesor_se[i,j],digits=8),"\t",A[i,j,m],"\t",format(amp_se[i,j,m],digits=8),"\t",format(PHI[i,j,m],digits=8),"\t",format(phi_se[i,j,m],digits=8),"\n")
              if (oneCycle[1]>0){   # print this cycle result if  oneCycle parameter set is>0
                ht<-10-(i+(j*htVar))+jht-(m-1)
                #print(ht)
                #  prints period and estimates for each period in a range
                text(c(-.4,1.5,2.7,3.3,4.3,5.1,6.2,7.2,8.2,9.2),c(ht,ht,ht,ht,ht,ht,ht,ht,ht,ht,ht),labels=c(paste(StartSpans[j]," - ",format(StartSpans[j]+Interval-1,nsmall=3)),format(Cycle[i,j,m],nsmall=2),format(P[i,j,m],digits=3,nsmall=3),format(PR[i,j,m],digits=5),format(M[i,j],digits=8), format(mesor_se[i,j],digits=6),format(A[i,j,m],digits=6),format(amp_se[i,j,m],digits=6),format(PHI[i,j,m],digits=6),format(phi_se[i,j,m],digits=6)),cex=cexVar,adj = c(0,0))
              }   #   format(F[i,j,m],digits=5),   removed 
            }  # end  Components loop
            if (Components>1){
              Magnitude[i,j]<-max(multi_Model_mag, na.rm=TRUE) - min(multi_Model_mag, na.rm=TRUE)
                Bathyphase[i,j]<- which.max(multi_Model_mag)-360
                bathyTime<- RefTime + (which.max(multi_Model_mag)*dTime*3600) 
                Orthophase[i,j]<- which.min(multi_Model_mag)-360
                orthoTime<- RefTime + (which.min(multi_Model_mag)*dTime*3600) 
              cat("ortho",Orthophase[i,j],"bathy",Bathyphase[i,j]) 
              cat("ortho",orthoTime,"bathy",bathyTime)
            
                if (i==1 && j==1){
                  par(mfrow=c(1,1),mar=c(4,4,4,1))
                  printXlab<-paste("Degrees, from times:",RefTimeString,"-",format(magTimeEnd,"%Y%m%d%H%M"),"(",1,"-",format(magDt,digits=3,nsmall=2),")","o",format(orthoTime,"%H:%M"),"b",format(bathyTime,"%H:%M"),"o",which.min(multi_Model_mag),"b",which.max(multi_Model_mag))
                  plot(x=c(1:360),y=multi_Model_mag,xlab=printXlab,ylab="mmHg",xlim=c(1,360))
                }
              ht<-9-(j*htVar)+jht
              #cat("F  ",multi_F,"P  ",multi_P[i,j],"PR  ",multi_PR[i,j])
            } 
            }  #  end check for matrix validity
            }   # end matrix cannot be inverted
          }    #  end i=Loops for loop

          if (oneCycle[1]==0){   # print cycle results here at end if  oneCycle parameter set 0

            ht<-9-(j*htVar)+jht        #8    5, 10, 11, 20, 15, 25
            # prints only the max for each spectrum of frequencies
            text(c(-.4,1.5,2.7,3.3,4.3,5.1,6.2,7.2,8.2,9.2),c(ht,ht,ht,ht,ht,ht,ht,ht,ht,ht,ht),labels=c(paste(StartSpans[j]," - ",format(StartSpans[j]+Interval-1,nsmall=3)),format(Cycle[maxAmp,j,Components],nsmall=2),format(P[maxAmp,j,Components],digits=3,nsmall=3),format(PR[maxAmp,j,Components],digits=5),format(M[maxAmp,j],digits=6), format(mesor_se[maxAmp,j],digits=6),format(A[maxAmp,j,Components],digits=6),format(amp_se[maxAmp,j,Components],digits=6),format(PHI[maxAmp,j,Components],digits=6),format(phi_se[maxAmp,j,Components],digits=6)),cex=cexVar,adj = c(0,0))
          }    #  end oneCycle
      }    #   END for (j in 1:Progression_end){  ,seq(from=magTime[1],by=dTime,to=tail(magTime,1))

    #  build title
    startDateP<-as.POSIXct(strptime(x=StartDate, format="%Y%m%d%H%M", tz=tz))
    endDateP<-as.POSIXct(strptime(x=EndDate, format="%Y%m%d%H%M", tz=tz))
    yTitle<-Y[y]

    if (header==TRUE){
      if (length(names(myHeader))==0){
        yTitle<-names(myHeader)[Y[y]]
      } 
    }
    titleMain<-paste('Column:',yTitle,',',yTitle,';   ',format(startDateP, "%Y-%m-%d %H:%M"),"--",format(endDateP, "%Y-%m-%d %H:%M"))       # graph title
    m<-Components
    if ((nrow(A)>1 || m>1) & ncol(A)>1 & GraphSet$HeatMap & Output$Graphs==TRUE){      #  
      par(mfrow=c(1,1),mar=c(4,4,3,4))
      if (m==1){
        mapx <- 1/Cycle[,1,1:m]                ###F    m is current component      rev(Cycle[1:(i),1,m]) 
        mapx2 <- c(1:RowCnt)
        ampz <- A[1:i,,m] 
      } else {
        mapx <- 1/Cycle[,1,1:m]
        mapx2 <- c(1:m) 
        ampz <- t(A[i,,1:m])
      }
      #browser()
      mapy <- StartSpans+StartTime+(Interval/2)      #  10*(1:ncol(A))  Adds Interval/2 so ticks are in middle of Interval
#       tickCnt<-MyData_hours/1440    # don't want too many ticks or they are not helpful, so reduce # ticks if needed
#       if (tickCnt>60){
#         findDivisor<-floor(tickCnt/60)
#       } else {
#         findDivisor<-floor(tickCnt)
#       }
      tickCnt<-format(seq(from=StartSpans[1]+StartTime+(Interval/2), to=MyData_hours+StartTime-(Interval/2),by=24),digits=4)
      if (Period$Set[1]==0){                      #  Set=0 means spectrum;
        #  y axis label
        HeatYlab<-paste('Period (hours): ',Period$Start,'/1, ',Period$Start, '/',1+FreqInc,' . . . to ',Period$Start,'/',format((1+((i-1)*FreqInc)),digits=4,nsmall=2),'=',format(cycle,digits=4,nsmall=2),", ","Harmonic increment:",FreqInc)
        HeatYlab2<-paste('Frequency (cycles/',Period$Start,'hour):  1, ',FreqInc+1,', . . . to ',format(RowCnt,digits=4,nsmall=1),", ","Harmonic increment:",FreqInc)
      }else {HeatYlab<-paste('Period (hours):  (At',Period["Set"],"periods)")
             HeatYlab2<-paste('Frequency (cycles/',Period$Start,'hour):  (At',Period["Set"],"periods)")
             mapx<-sort(mapx, decreasing=FALSE)    #  Set<>0, periods need to be sorted for proper graphing
      }

      lvlCnt<-10        # number of colors to use
      colCount<-100    #**  19 for hanning filter;  9 w/o filter
      Nscale<-((0:colCount)/colCount)
      Nbw<-gray(Nscale)
      HeatMain<-titleMain      # graph title
      HeatXlab<-paste('Time (hours from reference time,',RefTime,')\n','Interval length:',Interval,Units,'   Increment:',Increment,Units)     #  x axis label
      HeatLegend<-Nscale*max(A, na.rm=TRUE)      # use the number of levels needed, multiply by max, to get character labels for legend
      HeatLegend2<-c(HeatLegend,max(A, na.rm=TRUE)+max(A, na.rm=TRUE)/colCount)    # this is only needed for image breaks=HeatLegend2, where breaks needs to have one more element than legend
      HeatIndex<-rep(1:(colCount/lvlCnt),lvlCnt)
      HeatLegend3<-(HeatLegend2[HeatIndex==1])
      NbwFill<-(Nbw[HeatIndex==1])
      #HeatLegend<-format(HeatLegend2,digits=4)
      #zLim=c(0,30)
      if (Colors=="BW"){    #  begin Colors
        colorVar<-rev(Nbw)
        fillVar<-rev(NbwFill)
      }else {if (Colors=="Heat"){   
        colorVar<-rev(heat.colors(colCount))
        fillVar<-rev(heat.colors(lvlCnt+1))
      } else {colorVar<-rev(terrain.colors(colCount))
              fillVar<-rev(terrain.colors(lvlCnt+1))
             }   
      }#   end Color
      image(mapy, mapx, t(ampz), axes = FALSE, col=colorVar,main=HeatMain,ylab=HeatYlab,xlab=HeatXlab) #format(max(A, na.rm=TRUE)+max(A, na.rm=TRUE)*.2,digits=4)))      ###F    fill=rev(Nbw)
      contour(mapy, mapx, t(ampz),  axes = FALSE, nlevels=lvlCnt, labels=HeatLegend3, add=TRUE, drawlabels=FALSE)     #format(max(A, na.rm=TRUE)+max(A, na.rm=TRUE)*.2,digits=4)))      ###F    fill=rev(Nbw)
      #col controls the colors used in graph;  breaks controls the range of values for each color
      par(xpd=TRUE)  # Do not clip to the drawing area
      lambda <- .025
      #  legend controls the location, numbers and colors shown on the legend.  No relationship to actual unless same is used.
      legend(par("usr")[2],(par("usr")[4]-par("usr")[3])*.5,xjust = 0, yjust = 0,lwd=3, lty=1, title="Amplitude", legend=format(HeatLegend3,digits=4),fill=fillVar,text.width = strwidth("1.00"),cex=.5)                
      mtext('Progressive',side=1,line = 3, cex =.7, adj=1)
      #x axis corresponds to row number and the y axis to column number, with column 1 at the bottom, 
      #i.e. a 90 degree counter-clockwise rotation of the conventional printed layout of a matrix.
      axis(1, at = tickCnt)  
      axis(2, at = mapx,labels=format(1/mapx,digits=4) )      # 1/Cycle[1:(i),1]  
      #par(new=T)
      rightAxis<-c(1:RowCnt)
      axis(4, at = mapx,labels=format(mapx,digits=4))    #  cycles
      box()
      #  second plot with different Y axis.
      image(mapy, mapx2, t(ampz), axes = FALSE, col=colorVar,main=HeatMain,ylab=HeatYlab2,xlab=HeatXlab) #format(max(A, na.rm=TRUE)+max(A, na.rm=TRUE)*.2,digits=4)))      ###F    fill=rev(Nbw)
      contour(mapy, mapx2, t(ampz),  axes = FALSE, nlevels=lvlCnt, labels=HeatLegend3, add=TRUE, drawlabels=FALSE)     #format(max(A, na.rm=TRUE)+max(A, na.rm=TRUE)*.2,digits=4)))      ###F    fill=rev(Nbw)
      #col controls the colors used in graph;  breaks controls the range of values for each color
      par(xpd=TRUE)  # Do not clip to the drawing area
      lambda <- .025
      #  legend controls the location, numbers and colors shown on the legend.  No relationship to actual unless same is used.
      legend(par("usr")[2],(par("usr")[4]-par("usr")[3])*.5,xjust = 0, yjust = 0,lwd=3, lty=1, title="Amplitude", legend=format(HeatLegend3,digits=4),fill=fillVar,text.width = strwidth("1.00"),cex=.5)                
      mtext("Progressive",side=1,line = 3, cex =.7, adj=1)
      #x axis corresponds to row number and the y axis to column number, with column 1 at the bottom, 
      #i.e. a 90 degree counter-clockwise rotation of the conventional printed layout of a matrix.
      axis(1, at = tickCnt)  
      axis(2, at = 1:RowCnt)    #  cycles
      box()
    }

 if (length(which(GraphSet==T))>0 ) {    # if something set to be TRUE
    if (Output$Graphs==TRUE){           #  (m>1 || i>1) &&   GraphSet   Data MESOR Amp Phi NPts

      #layoutSize<-length(which(GraphSet==T))+1    #  need 2 more than number of graphs
      if (j>1){
        layoutSize <- 8
      } else layoutSize <- 7
      layout(matrix(c(1:layoutSize), layoutSize, 1, byrow = TRUE), 
             widths=c(1), heights=c(.5,rep(x=2,times=layoutSize-4),1,.5,.5))          #   c(figure#1, figure#2), 2 row, 1 col, ...
      par(mar=c(2,4,1,1), oma=c(2,0,1,0))         # set up margins
      
      if (FreqInc !=1){    #  display increment on y axis if other than 1
        Ylable_inc<- paste("; harmonic increment:",FreqInc,"cycles")       
      } else {Ylable_inc <-" "}

      if (j>1) {              #  J is time Interval(s) -- progression through Interval start to ends
        
        for (l in 1:m){
          for (k in 1:Loops){   #  i/k is frequency calculations for each time Interval
            # all cycles repeated, so using only list of cycles from first loop, if more than one loop
            cat(k,j,l,"\n")
            lineX<-StartSpans + StartTime + (Interval/2)      #   1:j   calculate halfway points for graphing each Interval
            if (m==1){      #    plotting when m==1, plot spans of progressive cosinor 
              mainX<-paste("Column ",Ycol,";  Period",Cycle[k,j,l],";   Time (",Units,") from reference date: ",RefTime)
            } else {      #     plotting when m>1 plots periods of each component
              mainX<-paste("Column ",Ycol,";  Component",Period$Set[l],Cycle[k,j,l],":   Time (",Units,") from reference date: ",RefTime)
            }
            
            plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # 1 blank area
            if (GraphSet$Data){    #2
              plot(MyData$time.hour[1:EndIdx],MyData[1:EndIdx,Ycol],type="l",ylab=paste("data"),main=mainX)   #  should not use newData
              ##if (GraphSet$Model){
              ##  plot(Time,Wsignal,ylab="cosinor model", type="l",axes=FALSE,bty='n')
              ##}   replace NAs in dd with 0:  dd[is.na(dd)]<-0
              } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area

            if (GraphSet$MESOR==T && !all(is.na(M[k,]))){   # 3 if all NA, which.max will fail
              plot(lineX,M[k,],type="l",ylab="MESOR",col=8)     # each Interval has a different MESOR
            } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area

            if (GraphSet$Amp && !all(is.na(A[k,,l]))){   # 4 if all NA, which.max will fail
              ampMax<-which.max(A[k,,l])
              ampMin<-which.min(A[k,,l])
              seMax<-which.max(amp_se[k,,l])
              seMin<-which.min(amp_se[k,,l])
              plot(lineX,A[k,,l],type="l",ylab="Amplitude",col=4,ylim=c((A[k,ampMin,l]-amp_se[k,seMin,l]),(A[k,ampMax,l]+amp_se[k,seMax,l])))
              points(lineX,(A[k,,l]+amp_se[k,,l]),type="p",pch=20,cex=.1,col=3)       # ,ylim=c(-360,0)
              points(lineX,(A[k,,l]-amp_se[k,,l]),type="p",pch=20,cex=.1,col=3)
            } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area

            if (GraphSet$Phi  && !all(is.na(PHI[k,,l]))){   # 5 if all NA, which.max will fail
              plot(lineX,PHI[k,,l],type="l",ylab="Acrophase",col=6,ylim=c(0,-360))

              points(lineX,(PHI[k,,l]+phi_se[k,,l]),type="p",pch=20,cex=.2,col=3)   
              points(lineX,(PHI[k,,l]-phi_se[k,,l]),type="p",pch=20,cex=.2,col=3)   
            } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area
            
            if (GraphSet$P){   # 6 for serial section only
              plot(lineX,P[k,,l],type="l",ylab="P Value",col=9,ylim=c(0,.5))
              abline(h=.05,lty="dashed")
            } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area
            
            if (GraphSet$NPts){   #7
              plot(lineX, nPts[k,],type="l",ylim=c(min(nPts[k,], na.rm=TRUE)-5,max(nPts[k,], na.rm=TRUE)+5) ,ylab=paste("N: Pts/Interval"))   # blank area
            } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area

            plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # 8 blank area
            mtext("Green dots = S.E. standard error",side=1,line = 2, cex =.8, adj=.7,col=3)
            #  if progressive, plot(lineX, X[k,]);   otherwise, plot(1:i,X[k,])  except variable always w/time
            # http://cran.r-project.org/doc/contrib/Lemon-kickstart/kr_addat.html
          }       # end K to Loops
        }   # end L to m
      }  else {    # && !any(errKey %in% Err)       # end if j>1, begin j==1,   more than 1 period, end if inversion error
        
        layout(matrix(c(1:layoutSize), layoutSize, 1, byrow = TRUE), 
               widths=c(1), heights=c(1,rep(2,layoutSize-2),1))          #   c(figure#1, figure#2), 2 row, 1 col, ...
        
        #  j==1 in several cases:  >1 period single cosinor or multiple cosiner;  progressive;  range progressive
        #  perhaps spectrum could be line graph, and inc<1 could be bar graphs
        #  if Inc<1, this section may be dropped out per GCG:  should not graph these -- may show bar graphs of Amp only
        CycleLen<-length(Cycle)
        cat("cycle",CycleLen)
        if (CycleLen>6){    #   if Cycle = 0 there is a long sequence, otherwise only a few
          titleLable2<-paste ("Periods:",format(Cycle[i,1,],digits=4,nsmall=2),' to ',format(Cycle[1,1,],digits=4,nsmall=2),Units, Ylable_inc)
          } else {
          titleLable2<- paste("Periods:",list(Cycle[,1,]),Units, Ylable_inc)
        }
        plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # 1 blank area
        if (GraphSet$Data){   #2, 3 cond
          plot(MyData$time.hour[StartIdx:EndIdx],newData,type="l",ylab=paste("data*",window),main=titleMain)
          plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # EXTRA blank area
          text(x=5.3,y=9.9, paste("Time (",Units,") from reference date: ",RefTime,cex=1.2))
          text(x=5.3,y=1.1,titleLable2,cex=1.2)
          abline(h=5,lty="longdash")
        } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area
        
        if (m>1 || i>1)  {
          if (GraphSet$MESOR==T && !all(is.na(M[CycleLen:1]))){   # 3 if all NA, min/max will fail
            plot(1/Cycle[CycleLen:1],M[CycleLen:1],type="l",ylab='MESOR2',col=8)
          } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area
  
          if (GraphSet$Amp && !all(is.na(A[CycleLen:1]))){   # 4 if all NA, min/max will fail
            AmpRange<-c((A[CycleLen:1]-amp_se[CycleLen:1]),(A[CycleLen:1]+amp_se[CycleLen:1]))
            plot(1/Cycle[CycleLen:1],A[CycleLen:1],type="l",ylab="Amplitude",col=4,ylim=c(min(AmpRange, na.rm=TRUE),max(AmpRange, na.rm=TRUE)))    #,ylim=c(0,(A[]+amp_se[])))
            points(1/Cycle[CycleLen:1],(A[CycleLen:1]+amp_se[CycleLen:1]),type="p",pch=20,cex=.6,col=3)   #,ylim=c(-360,0))   
            points(1/Cycle[CycleLen:1],(A[CycleLen:1]-amp_se[CycleLen:1]),type="p",pch=20,cex=.6,col=3)   #,ylim=c(-360,0))
            #lines(A[1,]+M[1,],type="l")
          } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area
  
          if (GraphSet$Phi && !all(is.na(PHI[CycleLen:1]))){   # 5 if all NA, min/max will fail
            plot(1/Cycle[CycleLen:1],PHI[CycleLen:1],type="l",ylab="Acrophase",col=6,ylim=c(0,-360))
            points(1/Cycle[CycleLen:1],(PHI[CycleLen:1]+phi_se[CycleLen:1]),type="p",pch=20,cex=.6,col=3)   
            points(1/Cycle[CycleLen:1],(PHI[CycleLen:1]-phi_se[CycleLen:1]),type="p",pch=20,cex=.6,col=3)   
            #plot(1/Cycle[CycleLen:1],PR[CycleLen:1],type="l",ylab="Percent Rhythm",col=6) 
          } else plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # blank area
          
          plot(1:10,1:10,type="n",ylab="",axes=FALSE)   # 6 blank area
          mtext("Green dots = S.E. standard error",side=1,line = 2, cex =.8, adj=.7,col=3)
        #}       # end K to Loops
      }  #  end m>1 || i>1  
      } # end else if j=1
    } # end if i>1 and Graphs == T
}  #  end LineSetGraph==T

    if(Output$Doc || Output$Dat){
      if (y==1){
        if (Output$Doc){
          #  may need to chg to wd:  http://www.r-statistics.com/2010/05/exporting-r-output-to-ms-word-with-r2wd-an-example-session/
          rtf<-RTF(fileName3,width=11,height=8.5,omi=c(.5,.25,.5,.25), font.size=11)
          output<-gsub(pattern='\b\b',replacement='\b',fileName3)
          paramMsg<-paste(output,"\n  TimeCol=",TimeCol,",  Ycol=",Ycol, ",  header=",header,"\n --  Periods=",Period["Set"],", Units=",Units, ",  Interval=",format(Interval,nsmall=3), ",  Increment=",format(Increment,nsmall=3), "\nPeriod$Start=",format(Period$Start,nsmall=3), ",  FreqInc=",format(FreqInc,nsmall=3), ",  Period$End=",format(Period$End,nsmall=3), "\nRefDateTime=",RefDateTime, ", StartTime=",format(StartTime,nsmall=3),", EndTime=",format(EndTime,nsmall=3),"Percent of missing (blank) sample values: %",missingData,"\n",functionName,"\n")
          addParagraph(rtf,paramMsg)
          addParagraph(rtf,errMsg)
          #errMsg<-""  
        }
        PRdigits<-2
        PMAdigits<-3
        PHIdigits<-1
        fileoutCosX <- paste(fileName,window,thisTime,functionName,"Cos.dat",sep="")
      }
      printP<-format(P,digits=4,nsmall=PMAdigits,scientific=FALSE)
      printP[P<.0005]<-c("<.001")
      sPtsX<-array(data=sPts,c(i,j,m))
      MX<-array(data=M,c(i,j,m))
      mesor_seX<-array(data=mesor_se,c(i,j,m))
      timeX<-array(data=time,c(i,j,m))
      yVarX<-array(data=yVar,c(i,j,m))
      hoursX<-array(data=hours,c(i,j,m))
      #ErrX<-array(data=Err,c(i,j,m))

      if(Output$Doc){
        if (window=="noTaper"){
          addParagraph(rtf,"\n\n        Rhythmometric Summary")
        } else {
          addParagraph(rtf,"\n\n        Rhythmometric Summary of filtered data")
        }
        if (Components>1){
          for (n in 1:Progression_end){
            addParagraph(rtf,paste("\n column",Y[y],":    MESOR=",format(M[,n],digits=2*PRdigits,nsmall=PMAdigits),"     MESOR s.e.=",format(mesor_se[,n],digits=2*PRdigits,nsmall=PMAdigits)))
            addParagraph(rtf,paste("\n           Time Pts=",time[,n],"     hours from RefDateTime=",hours[,n],"     #Pts","Period in hrs=",sPts[,n]))
            mat<-data.frame(matrix(c(Err[,n,],format(Cycle[,n,],nsmall=PRdigits),format(newPR[,n,],digits=2*PRdigits,nsmall=PRdigits,scientific=FALSE),printP[,n,],testPg[,n,],testPb[,n,],format(A[,n,],digits=2*PRdigits,nsmall=PMAdigits),format(amp_se[,n,],digits=2*PRdigits,nsmall=PMAdigits),format(PHI[,n,],digits=2*PHIdigits,nsmall=PHIdigits,scientific=FALSE),format(phi_se[,n,],digits=2*PHIdigits,nsmall=PHIdigits)),ncol=11))  #,dimnames=list(row_names,colName))
            names(mat)<-c(" ","Period in Hrs","newPR","P","P(Ger)","P(Bin)","Amp","Amp SE","PHI","PHI SE") # format column names
            #addTable(rtf,mat,font.size=11,row.names=FALSE,NA.string="--",col.widths=c(.6,.3,1.3,1.05,.75,.6,.75,.6,.75,.75,.6,.6,.6,.6,.6))
            # changed from above to below after a long time of not running multis  --  too many output values in col.widths vector
            addTable(rtf,mat,font.size=11,row.names=FALSE,NA.string="--",col.widths=c(.6,.3,1.05,.75,.6,.75,.6,.75,.75,.6))
            printP_multi<-format(multi_P[i,n],digits=4,nsmall=PMAdigits)
            printP_multi[multi_P[i,n]<.0005]<-c("<.001")
            addParagraph(rtf,paste("\nLCM",LCM,"  Overall:   PR=",format(multi_PR[i,n],digits=2*PRdigits,nsmall=PMAdigits,scientific=FALSE),"      P=",printP_multi,"       Magnitude=",Magnitude[i,n],"        Orthophase",Orthophase[i,n],"        Bathyphase=",Bathyphase[i,n],"\n\n"))
          }   #   End progressive where Components>1
          } else {    #  Components =1
            
            mat<-data.frame(matrix(c(Err,yVar,time,hours,sPts,format(Cycle,nsmall=PRdigits),format(newPR,digits=2*PRdigits,nsmall=PRdigits,scientific=FALSE),printP,format(M,digits=2*PRdigits,nsmall=PMAdigits),format(mesor_se,digits=2*PRdigits,nsmall=PMAdigits),format(A,digits=2*PRdigits,nsmall=PMAdigits),format(amp_se,digits=2*PRdigits,nsmall=PMAdigits),format(PHI,digits=2*PHIdigits,nsmall=PHIdigits,scientific=FALSE),format(phi_se,digits=2*PHIdigits,nsmall=PHIdigits)),ncol=14))  #,dimnames=list(row_names,colName))
            #  commented out when doing trials of non-invertible matrix
            #mat<-data.frame(matrix(c(yVarX,timeX,hoursX,sPtsX,format(Cycle,nsmall=PRdigits),format(PR,digits=2*PRdigits,nsmall=PRdigits),format(newPR,digits=2*PRdigits,nsmall=PRdigits),printP,format(MX,digits=2*PRdigits,nsmall=PMAdigits),format(mesor_seX,digits=2*PRdigits,nsmall=PMAdigits),format(A,digits=2*PRdigits,nsmall=PMAdigits),format(amp_se,digits=2*PRdigits,nsmall=PMAdigits),format(PHI,digits=2*PHIdigits,nsmall=PHIdigits),format(phi_se,digits=2*PHIdigits,nsmall=PHIdigits)),ncol=14))  #,dimnames=list(row_names,colName))
            names(mat)<-c("Err","Y","Time Pts","hours from RefDateTime","#Pts","Period in hrs","newPR","P","Mesor","Mesor SE","Amp","Amp SE","PHI","PHI SE") # format column names
            addTable(rtf,mat,font.size=11,row.names=FALSE,NA.string="--",col.widths=c(.6,.3,1.3,1.05,.75,.6,.6,.75,.75,.6,.6,.6,.6,.6))   # col.widths=c(1,0.5,0.5,0.5,0.5)
          }
        #  summary of data
        if (window=="noTaper"){
          addParagraph(rtf,"\n\n        Data Summary")
        } else {
          addParagraph(rtf,"\n\n        Data Summary of filtered data")
        }

        if (Components>1){
          #for (n in 1:Progression_end){
            # changed 1-->n below, without testing this:  matS<-data.frame(matrix(cbind(ErrX[y,1,m],yVarX[y,1,m],timeX[y,1,m],hoursX[y,1,m],sPtsX[y,1,m],format(sumN[y,1:Progression_end],nsmall=PRdigits),format(sumLow[y,1:Progression_end],nsmall=PRdigits),format(sumHi[y,1:Progression_end],nsmall=PRdigits),format(sumMean[y,1:Progression_end],nsmall=PRdigits),format(sumMedian[y,1:Progression_end],nsmall=PRdigits),format(sumMode[y,1:Progression_end],nsmall=PRdigits),format(sumT[y,1:Progression_end],nsmall=PRdigits),format(sumSD[y,1:Progression_end],nsmall=PRdigits)),ncol=13))  #,dimnames=list(row_names,colName))
            # removed for loop because we only need one instance of each interval for the data summary for progressives with multiple components
            matS<-data.frame(matrix(cbind(Err[1,,m],yVarX[1,,m],timeX[1,,m],hoursX[1,,m],sPtsX[1,,m],format(sumN[y,1:Progression_end],nsmall=PRdigits),format(sumLow[y,1:Progression_end],nsmall=PRdigits),format(sumHi[y,1:Progression_end],nsmall=PRdigits),format(sumMean[y,1:Progression_end],nsmall=PRdigits),format(sumMedian[y,1:Progression_end],nsmall=PRdigits),format(sumMode[y,1:Progression_end],nsmall=PRdigits),format(sumT[y,1:Progression_end],nsmall=PRdigits),format(sumSD[y,1:Progression_end],nsmall=PRdigits)),ncol=13))  #,dimnames=list(row_names,colName))
            names(matS)<-c(Err,"Y","Time Pts","hours from RefDateTime","#Pts","N","Low","High","Mean","Median","Mode","dT","s.d.") # format column names
            addTable(rtf,matS,font.size=11,row.names=FALSE,NA.string="--",col.widths=c(.6,.3,1.3,1.05,1.4,.6,.6,.6,.75,.75,.6,.75,.75))
          
          #}   #   End progressive where Components>1
        } else {    #  Components =1
          #if (Progression_end>1){ 
            # y,1,m is needed for progressive because we only need one instance of each interval 
           #matS<-data.frame(matrix(cbind(Err[1,1,m],yVarX[1,1,m],timeX[1,1,m],hoursX[1,1,m],sPtsX[1,1,m],format(sumN[y,1:Progression_end],nsmall=PRdigits),format(sumLow[y,1:Progression_end],nsmall=PRdigits),format(sumHi[y,1:Progression_end],nsmall=PRdigits),format(sumMean[y,1:Progression_end],nsmall=PRdigits),format(sumMedian[y,1:Progression_end],nsmall=PRdigits),format(sumMode[y,1:Progression_end],nsmall=PRdigits),format(sumT[y,1:Progression_end],nsmall=PRdigits),format(sumSD[y,1:Progression_end],nsmall=PRdigits)),ncol=13))  #,dimnames=list(row_names,colName))
          #} else {
            # (used ,,m for non progressive)
            matS<-data.frame(matrix(cbind(Err[1,,m],yVarX[1,,m],timeX[1,,m],hoursX[1,,m],sPtsX[1,,m],format(sumN[y,1:Progression_end],nsmall=PRdigits),format(sumLow[y,1:Progression_end],nsmall=PRdigits),format(sumHi[y,1:Progression_end],nsmall=PRdigits),format(sumMean[y,1:Progression_end],nsmall=PRdigits),format(sumMedian[y,1:Progression_end],nsmall=PRdigits),format(sumMode[y,1:Progression_end],nsmall=PRdigits),format(sumT[y,1:Progression_end],nsmall=PRdigits),format(sumSD[y,1:Progression_end],nsmall=PRdigits)),ncol=13))  #,dimnames=list(row_names,colName))
           # }
          
          names(matS)<-c("Err","Y","Time Pts","hours from RefDateTime","#Pts","N","Low","High","Mean","Median","Mode","dT","s.d.") # format column names
          addTable(rtf,matS,font.size=11,row.names=FALSE,NA.string="--",col.widths=c(.6,.3,1.3,1.05,1.4,.6,.6,.6,.75,.75,.6,.75,.75))
        }  # End Components = 1
        if (Progression_end>1){     #  if there is more than one Interval, print final row:  full data summary 
          # non progressive and progressive!  matS<-data.frame(matrix(cbind(Err,Y,paste(StartDate,"-",EndDate),format(MyData_hours,digits=PRdigits,nsmall=PRdigits),paste(StartIndex[1],"-",tail(EndIndex,1)),format(sumN,digits=PRdigits,nsmall=PRdigits),format(sumLow[,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumHi[,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMean[,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMedian[,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMode[,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumT[,1:Progression_end],nsmall=PRdigits),format(sumSD[,Progression_end+1],digits=PRdigits,nsmall=PRdigits)),ncol=13))
          #matS<-data.frame(matrix(cbind(Err[y,1,m],yVarX[y,1,m],timeX[y,1,m],hoursX[y,1,m],sPtsX[y,1,m],format(sumN,nsmall=PRdigits),format(sumLow[y,1:Progression_end+1],nsmall=PRdigits),format(sumHi[y,1:Progression_end+1],nsmall=PRdigits),format(sumMean[y,1:Progression_end+1],nsmall=PRdigits),format(sumMedian[y,1:Progression_end+1],nsmall=PRdigits),format(sumMode[y,1:Progression_end+1],nsmall=PRdigits),format(sumT[y,1:Progression_end+1],nsmall=PRdigits),format(sumSD[y,1:Progression_end+1],nsmall=PRdigits)),ncol=13))  #,dimnames=list(row_names,colName))
          matS<-data.frame(matrix(cbind("",Ycol,paste(StartDate,"-",EndDate),format(MyData_hours,digits=PRdigits,nsmall=PRdigits),paste(StartIndex[1],"-",tail(EndIndex,1)),format(sumN[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumLow[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumHi[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMean[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMedian[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMode[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumT[y,Progression_end+1],nsmall=PRdigits),format(sumSD[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits)),ncol=13))
          names(matS)<-c("Err","Y","Total Time","Total Hours","Total Pts#","N","Low","High","Mean","Median","Mode","dT","s.d.")
          addTable(rtf,matS,font.size=11,row.names=FALSE,NA.string="--",col.widths=c(.6,.3,1.3,1.05,1.4,.6,.6,.6,.75,.75,.6,.75,.75))
        }
      }     # end RTF
      #####################################################################################################
      #                                                                                                   #
      #        print the Rhythmometric Summary for each Period and Y;   sumXxx are summary variables      #
      #                                   Each Y data set has a TOTALS row, and a row for each interval   #
      #                Non-progressives:  multi Y, multi component, multi-period all are done the same:   #
      #                                   (y,1+1) holds the summary variables for non-progressives        #
      #                                   (y,1) is the same as (y,1+1) so is not used                     #
      #                Progressives:  progressives have one summary for each span, and a total row        #
      #                                   (y,progression_end) holds the summary variables                 #
      #                                   (y,progression_end+1) holds the summary variables totals        #
      #                                                                                                   #
      #        Columns preceeding data are used in later data processing to re-sort and re-order          #
      #                                                                                                   #
      #####################################################################################################
      
      if (Output$Dat){    #  figure out how to write without closing; and then how to write without opening;  Time, Pts, hours, #Pts are not working
        sPtsX2<-sub(pattern="\n",replacement=" ",x=sPtsX)   # this didn't work on matrix
        jLabel<-c(1:Progression_end)
        if (Components>1){
          PRX<-array(data=multi_PR,c(i,j,m))
          printP_multi<-format(multi_P,digits=4,nsmall=PMAdigits)
          printP_multi[multi_P<.0005]<-c("<.001")
          PX<-array(data=printP_multi,c(i,j,m))
          MagX<-array(data=Magnitude,c(i,j,m))
          BathyX<-array(data=Bathyphase,c(i,j,m))
          OrthoX<-array(data=Orthophase,c(i,j,m))
          #LCM,format(multi_PR[i,n],digits=2*PRdigits,nsmall=PMAdigits,scientific=FALSE),printP_multi
          #"       Magnitude=",Magnitude[i,n],"        Orthophase",Orthophase[i,n],"        Bathyphase=",Bathyphase[i,n],"\n\n"))
          datMat<-cbind("R",Err,fileName6,jLabel,yVarX,sub("\n",timeX,replacement="",fixed=TRUE),hoursX,sPtsX2,format(Cycle,nsmall=PRdigits),format(newPR,digits=2*PRdigits,nsmall=PRdigits,scientific=FALSE),printP,format(MX,digits=2*PRdigits,nsmall=PMAdigits),format(mesor_seX,digits=2*PRdigits,nsmall=PMAdigits),format(A,digits=2*PRdigits,nsmall=PMAdigits),format(amp_se,digits=2*PRdigits,nsmall=PMAdigits),format(PHI,digits=2*PHIdigits,nsmall=PHIdigits,scientific=FALSE),format(phi_se,digits=2*PHIdigits,nsmall=PHIdigits),LCM,PRX,PX,MagX,OrthoX,BathyX)
          datMatNames<-c("Hdr","Err","file","j","Y","Time Pts","hours from RefDateTime","#Pts","Period in Hrs","newPR","P","Mesor","Mesor SE","Amp","Amp SE","PHI","PHI SE","Total:  LCM","PR","P","Magnitude", "Orthophase","Bathyphase")
        } else{
          datMat<-cbind("R",Err,fileName6,jLabel,yVarX,sub("\n",timeX,replacement="",fixed=TRUE),hoursX,sPtsX2,format(Cycle,nsmall=PRdigits),format(newPR,digits=2*PRdigits,nsmall=PRdigits,scientific=FALSE),printP,format(MX,digits=2*PRdigits,nsmall=PMAdigits),format(mesor_seX,digits=2*PRdigits,nsmall=PMAdigits),format(A,digits=2*PRdigits,nsmall=PMAdigits),format(amp_se,digits=2*PRdigits,nsmall=PMAdigits),format(PHI,digits=2*PHIdigits,nsmall=PHIdigits,scientific=FALSE),format(phi_se,digits=2*PHIdigits,nsmall=PHIdigits))
          datMatNames<-c("Hdr","Err","file","j","Y","Time Pts","hours from RefDateTime","#Pts","Period in Hrs","newPR","P","Mesor","Mesor SE","Amp","Amp SE","PHI","PHI SE")
        }
    
        # Needs to have [y,,x]    call12rhcsingle.R   --  added 1 to first 5 for Yoshiko/callYW8714.R
        datMatS<-cbind("S",Err[1,,m],fileName6,jLabel,yVarX[1,,m],sub("\n",timeX[1,,m],replacement="",fixed=TRUE),hoursX[1,,m],sPtsX2[1,,m],format(sumN[y,1:Progression_end],nsmall=PRdigits),format(sumLow[y,1:Progression_end],digits=PRdigits,nsmall=PRdigits),format(sumHi[y,1:Progression_end],digits=PRdigits,nsmall=PRdigits),format(sumMean[y,1:Progression_end],digits=PRdigits,nsmall=PRdigits),format(sumMedian[y,1:Progression_end],digits=PRdigits,nsmall=PRdigits),format(sumMode[y,1:Progression_end],digits=PRdigits,nsmall=PRdigits),format(sumT[y,1:Progression_end],nsmall=PRdigits),format(sumSD[y,1:Progression_end],digits=PRdigits,nsmall=PRdigits))
        datMatNamesS<-c("Hdr","Err","file","j","Y","Time Pts","hours from RefDateTime","#Pts","N","Low","High","Mean","Median","Mode","dT","SD")
        dimnames(datMat)<-NULL
        dimnames(datMatS)<-NULL
        #eol = "\r\n" will produce Windows' line endings on a Unix-alike OS, and eol = "\r" will produce files as expected by Excel:mac 2004.
        # write Rhythmometric data to file, including header if y==1 
        if (y==1){
          write.table(datMat,file=fileoutCosX,append=FALSE,quote=FALSE,sep="\t",row.names=FALSE,col.names=datMatNames)
        } else {
          write.table(datMat,file=fileoutCosX,append=TRUE,quote=FALSE,sep="\t",row.names=FALSE,col.names=datMatNames)
        }
        #  write Data Summary Data (each Y has a TOTAL data summary, and each progressive span has a data summary)
        write.table(datMatS,file=fileoutCosX,append=TRUE,quote=FALSE,sep="\t",row.names=FALSE,col.names=datMatNamesS)
        if (Progression_end>1){     #  if there is more than one Interval, print full data summary  
          # (chgned from Y to Ycol when progressive, multi-period, multi-y, single comp)
          datMatS<-cbind("STot","",fileName6,"",Ycol,paste(StartDate,"-",EndDate),format(MyData_hours,digits=PRdigits,nsmall=PRdigits),paste(StartIndex[1],"-",tail(EndIndex,1)),format(sumN[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumLow[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumHi[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMean[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMedian[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumMode[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),format(sumT[y,Progression_end+1],nsmall=PRdigits),format(sumSD[y,Progression_end+1],digits=PRdigits,nsmall=PRdigits),"\n")
          datMatNamesS<-c("Hdr"," ","file"," ","Y","Total Time","Total Hours","Total Pts#","N","Low","High","Mean","Median","Mode","dT","SD"," ")
          dimnames(datMatS)<-NULL

          write.table(datMatS,file=fileoutCosX,append=TRUE,quote=FALSE,sep="\t",row.names=FALSE,col.names=datMatNamesS)
         }
        
      }   # end Dat
      # } 
    }   # end RTF or Dat

    if (Output$Doc){
      addNewLine(rtf,4)
      if (y==Ys_end && j==Progression_end){
        addTable(rtf,as.data.frame(keyMsgs),row.names="Key:  Err column decode.")
        addNewLine(rtf,4)
        addPageBreak(rtf)
        addSessionInfo(rtf)
      } else {
        Err <- array(data="",c(RowCnt, Progression_end,Components))
      }
      done(rtf)                    # writes and closes the file
    }
  }    #    END for (y in 1:Ys_end){

closeOutput(file=fileName4,output=Output,console=Console,opar=opar)
}
